# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe._webhook_endpoint import WebhookEndpoint
    from stripe.params._webhook_endpoint_create_params import (
        WebhookEndpointCreateParams,
    )
    from stripe.params._webhook_endpoint_delete_params import (
        WebhookEndpointDeleteParams,
    )
    from stripe.params._webhook_endpoint_list_params import (
        WebhookEndpointListParams,
    )
    from stripe.params._webhook_endpoint_retrieve_params import (
        WebhookEndpointRetrieveParams,
    )
    from stripe.params._webhook_endpoint_update_params import (
        WebhookEndpointUpdateParams,
    )


class WebhookEndpointService(StripeService):
    def delete(
        self,
        webhook_endpoint: str,
        params: Optional["WebhookEndpointDeleteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "WebhookEndpoint":
        """
        You can also delete webhook endpoints via the [webhook endpoint management](https://dashboard.stripe.com/account/webhooks) page of the Stripe dashboard.
        """
        return cast(
            "WebhookEndpoint",
            self._request(
                "delete",
                "/v1/webhook_endpoints/{webhook_endpoint}".format(
                    webhook_endpoint=sanitize_id(webhook_endpoint),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def delete_async(
        self,
        webhook_endpoint: str,
        params: Optional["WebhookEndpointDeleteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "WebhookEndpoint":
        """
        You can also delete webhook endpoints via the [webhook endpoint management](https://dashboard.stripe.com/account/webhooks) page of the Stripe dashboard.
        """
        return cast(
            "WebhookEndpoint",
            await self._request_async(
                "delete",
                "/v1/webhook_endpoints/{webhook_endpoint}".format(
                    webhook_endpoint=sanitize_id(webhook_endpoint),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        webhook_endpoint: str,
        params: Optional["WebhookEndpointRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "WebhookEndpoint":
        """
        Retrieves the webhook endpoint with the given ID.
        """
        return cast(
            "WebhookEndpoint",
            self._request(
                "get",
                "/v1/webhook_endpoints/{webhook_endpoint}".format(
                    webhook_endpoint=sanitize_id(webhook_endpoint),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        webhook_endpoint: str,
        params: Optional["WebhookEndpointRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "WebhookEndpoint":
        """
        Retrieves the webhook endpoint with the given ID.
        """
        return cast(
            "WebhookEndpoint",
            await self._request_async(
                "get",
                "/v1/webhook_endpoints/{webhook_endpoint}".format(
                    webhook_endpoint=sanitize_id(webhook_endpoint),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        webhook_endpoint: str,
        params: Optional["WebhookEndpointUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "WebhookEndpoint":
        """
        Updates the webhook endpoint. You may edit the url, the list of enabled_events, and the status of your endpoint.
        """
        return cast(
            "WebhookEndpoint",
            self._request(
                "post",
                "/v1/webhook_endpoints/{webhook_endpoint}".format(
                    webhook_endpoint=sanitize_id(webhook_endpoint),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        webhook_endpoint: str,
        params: Optional["WebhookEndpointUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "WebhookEndpoint":
        """
        Updates the webhook endpoint. You may edit the url, the list of enabled_events, and the status of your endpoint.
        """
        return cast(
            "WebhookEndpoint",
            await self._request_async(
                "post",
                "/v1/webhook_endpoints/{webhook_endpoint}".format(
                    webhook_endpoint=sanitize_id(webhook_endpoint),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def list(
        self,
        params: Optional["WebhookEndpointListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[WebhookEndpoint]":
        """
        Returns a list of your webhook endpoints.
        """
        return cast(
            "ListObject[WebhookEndpoint]",
            self._request(
                "get",
                "/v1/webhook_endpoints",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["WebhookEndpointListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[WebhookEndpoint]":
        """
        Returns a list of your webhook endpoints.
        """
        return cast(
            "ListObject[WebhookEndpoint]",
            await self._request_async(
                "get",
                "/v1/webhook_endpoints",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "WebhookEndpointCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "WebhookEndpoint":
        """
        A webhook endpoint must have a url and a list of enabled_events. You may optionally specify the Boolean connect parameter. If set to true, then a Connect webhook endpoint that notifies the specified url about events from all connected accounts is created; otherwise an account webhook endpoint that notifies the specified url only about events from your account is created. You can also create webhook endpoints in the [webhooks settings](https://dashboard.stripe.com/account/webhooks) section of the Dashboard.
        """
        return cast(
            "WebhookEndpoint",
            self._request(
                "post",
                "/v1/webhook_endpoints",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "WebhookEndpointCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "WebhookEndpoint":
        """
        A webhook endpoint must have a url and a list of enabled_events. You may optionally specify the Boolean connect parameter. If set to true, then a Connect webhook endpoint that notifies the specified url about events from all connected accounts is created; otherwise an account webhook endpoint that notifies the specified url only about events from your account is created. You can also create webhook endpoints in the [webhooks settings](https://dashboard.stripe.com/account/webhooks) section of the Dashboard.
        """
        return cast(
            "WebhookEndpoint",
            await self._request_async(
                "post",
                "/v1/webhook_endpoints",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._application_fee_refund import ApplicationFeeRefund
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params._application_fee_refund_create_params import (
        ApplicationFeeRefundCreateParams,
    )
    from stripe.params._application_fee_refund_list_params import (
        ApplicationFeeRefundListParams,
    )
    from stripe.params._application_fee_refund_retrieve_params import (
        ApplicationFeeRefundRetrieveParams,
    )
    from stripe.params._application_fee_refund_update_params import (
        ApplicationFeeRefundUpdateParams,
    )


class ApplicationFeeRefundService(StripeService):
    def retrieve(
        self,
        fee: str,
        id: str,
        params: Optional["ApplicationFeeRefundRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ApplicationFeeRefund":
        """
        By default, you can see the 10 most recent refunds stored directly on the application fee object, but you can also retrieve details about a specific refund stored on the application fee.
        """
        return cast(
            "ApplicationFeeRefund",
            self._request(
                "get",
                "/v1/application_fees/{fee}/refunds/{id}".format(
                    fee=sanitize_id(fee),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        fee: str,
        id: str,
        params: Optional["ApplicationFeeRefundRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ApplicationFeeRefund":
        """
        By default, you can see the 10 most recent refunds stored directly on the application fee object, but you can also retrieve details about a specific refund stored on the application fee.
        """
        return cast(
            "ApplicationFeeRefund",
            await self._request_async(
                "get",
                "/v1/application_fees/{fee}/refunds/{id}".format(
                    fee=sanitize_id(fee),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        fee: str,
        id: str,
        params: Optional["ApplicationFeeRefundUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ApplicationFeeRefund":
        """
        Updates the specified application fee refund by setting the values of the parameters passed. Any parameters not provided will be left unchanged.

        This request only accepts metadata as an argument.
        """
        return cast(
            "ApplicationFeeRefund",
            self._request(
                "post",
                "/v1/application_fees/{fee}/refunds/{id}".format(
                    fee=sanitize_id(fee),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        fee: str,
        id: str,
        params: Optional["ApplicationFeeRefundUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ApplicationFeeRefund":
        """
        Updates the specified application fee refund by setting the values of the parameters passed. Any parameters not provided will be left unchanged.

        This request only accepts metadata as an argument.
        """
        return cast(
            "ApplicationFeeRefund",
            await self._request_async(
                "post",
                "/v1/application_fees/{fee}/refunds/{id}".format(
                    fee=sanitize_id(fee),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def list(
        self,
        id: str,
        params: Optional["ApplicationFeeRefundListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[ApplicationFeeRefund]":
        """
        You can see a list of the refunds belonging to a specific application fee. Note that the 10 most recent refunds are always available by default on the application fee object. If you need more than those 10, you can use this API method and the limit and starting_after parameters to page through additional refunds.
        """
        return cast(
            "ListObject[ApplicationFeeRefund]",
            self._request(
                "get",
                "/v1/application_fees/{id}/refunds".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        id: str,
        params: Optional["ApplicationFeeRefundListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[ApplicationFeeRefund]":
        """
        You can see a list of the refunds belonging to a specific application fee. Note that the 10 most recent refunds are always available by default on the application fee object. If you need more than those 10, you can use this API method and the limit and starting_after parameters to page through additional refunds.
        """
        return cast(
            "ListObject[ApplicationFeeRefund]",
            await self._request_async(
                "get",
                "/v1/application_fees/{id}/refunds".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        id: str,
        params: Optional["ApplicationFeeRefundCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ApplicationFeeRefund":
        """
        Refunds an application fee that has previously been collected but not yet refunded.
        Funds will be refunded to the Stripe account from which the fee was originally collected.

        You can optionally refund only part of an application fee.
        You can do so multiple times, until the entire fee has been refunded.

        Once entirely refunded, an application fee can't be refunded again.
        This method will raise an error when called on an already-refunded application fee,
        or when trying to refund more money than is left on an application fee.
        """
        return cast(
            "ApplicationFeeRefund",
            self._request(
                "post",
                "/v1/application_fees/{id}/refunds".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        id: str,
        params: Optional["ApplicationFeeRefundCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ApplicationFeeRefund":
        """
        Refunds an application fee that has previously been collected but not yet refunded.
        Funds will be refunded to the Stripe account from which the fee was originally collected.

        You can optionally refund only part of an application fee.
        You can do so multiple times, until the entire fee has been refunded.

        Once entirely refunded, an application fee can't be refunded again.
        This method will raise an error when called on an already-refunded application fee,
        or when trying to refund more money than is left on an application fee.
        """
        return cast(
            "ApplicationFeeRefund",
            await self._request_async(
                "post",
                "/v1/application_fees/{id}/refunds".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class CreditBalanceSummaryRetrieveParams(RequestOptions):
    customer: str
    """
    The customer for which to fetch credit balance summary.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    filter: "CreditBalanceSummaryRetrieveParamsFilter"
    """
    The filter criteria for the credit balance summary.
    """


class CreditBalanceSummaryRetrieveParamsFilter(TypedDict):
    applicability_scope: NotRequired[
        "CreditBalanceSummaryRetrieveParamsFilterApplicabilityScope"
    ]
    """
    The billing credit applicability scope for which to fetch credit balance summary.
    """
    credit_grant: NotRequired[str]
    """
    The credit grant for which to fetch credit balance summary.
    """
    type: Literal["applicability_scope", "credit_grant"]
    """
    Specify the type of this filter.
    """


class CreditBalanceSummaryRetrieveParamsFilterApplicabilityScope(TypedDict):
    price_type: NotRequired[Literal["metered"]]
    """
    The price type that credit grants can apply to. We currently only support the `metered` price type. Cannot be used in combination with `prices`.
    """
    prices: NotRequired[
        List["CreditBalanceSummaryRetrieveParamsFilterApplicabilityScopePrice"]
    ]
    """
    A list of prices that the credit grant can apply to. We currently only support the `metered` prices. Cannot be used in combination with `price_type`.
    """


class CreditBalanceSummaryRetrieveParamsFilterApplicabilityScopePrice(
    TypedDict,
):
    id: str
    """
    The price ID this credit grant should apply to.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._payment_link import PaymentLink
    from stripe._payment_link_line_item_service import (
        PaymentLinkLineItemService,
    )
    from stripe._request_options import RequestOptions
    from stripe.params._payment_link_create_params import (
        PaymentLinkCreateParams,
    )
    from stripe.params._payment_link_list_params import PaymentLinkListParams
    from stripe.params._payment_link_retrieve_params import (
        PaymentLinkRetrieveParams,
    )
    from stripe.params._payment_link_update_params import (
        PaymentLinkUpdateParams,
    )

_subservices = {
    "line_items": [
        "stripe._payment_link_line_item_service",
        "PaymentLinkLineItemService",
    ],
}


class PaymentLinkService(StripeService):
    line_items: "PaymentLinkLineItemService"

    def __init__(self, requestor):
        super().__init__(requestor)

    def __getattr__(self, name):
        try:
            import_from, service = _subservices[name]
            service_class = getattr(
                import_module(import_from),
                service,
            )
            setattr(
                self,
                name,
                service_class(self._requestor),
            )
            return getattr(self, name)
        except KeyError:
            raise AttributeError()

    def list(
        self,
        params: Optional["PaymentLinkListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[PaymentLink]":
        """
        Returns a list of your payment links.
        """
        return cast(
            "ListObject[PaymentLink]",
            self._request(
                "get",
                "/v1/payment_links",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["PaymentLinkListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[PaymentLink]":
        """
        Returns a list of your payment links.
        """
        return cast(
            "ListObject[PaymentLink]",
            await self._request_async(
                "get",
                "/v1/payment_links",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "PaymentLinkCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentLink":
        """
        Creates a payment link.
        """
        return cast(
            "PaymentLink",
            self._request(
                "post",
                "/v1/payment_links",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "PaymentLinkCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentLink":
        """
        Creates a payment link.
        """
        return cast(
            "PaymentLink",
            await self._request_async(
                "post",
                "/v1/payment_links",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        payment_link: str,
        params: Optional["PaymentLinkRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentLink":
        """
        Retrieve a payment link.
        """
        return cast(
            "PaymentLink",
            self._request(
                "get",
                "/v1/payment_links/{payment_link}".format(
                    payment_link=sanitize_id(payment_link),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        payment_link: str,
        params: Optional["PaymentLinkRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentLink":
        """
        Retrieve a payment link.
        """
        return cast(
            "PaymentLink",
            await self._request_async(
                "get",
                "/v1/payment_links/{payment_link}".format(
                    payment_link=sanitize_id(payment_link),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        payment_link: str,
        params: Optional["PaymentLinkUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentLink":
        """
        Updates a payment link.
        """
        return cast(
            "PaymentLink",
            self._request(
                "post",
                "/v1/payment_links/{payment_link}".format(
                    payment_link=sanitize_id(payment_link),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        payment_link: str,
        params: Optional["PaymentLinkUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentLink":
        """
        Updates a payment link.
        """
        return cast(
            "PaymentLink",
            await self._request_async(
                "post",
                "/v1/payment_links/{payment_link}".format(
                    payment_link=sanitize_id(payment_link),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class SetupIntentCreateParams(RequestOptions):
    attach_to_self: NotRequired[bool]
    """
    If present, the SetupIntent's payment method will be attached to the in-context Stripe Account.

    It can only be used for this Stripe Account's own money movement flows like InboundTransfer and OutboundTransfers. It cannot be set to true when setting up a PaymentMethod for a Customer, and defaults to false when attaching a PaymentMethod to a Customer.
    """
    automatic_payment_methods: NotRequired[
        "SetupIntentCreateParamsAutomaticPaymentMethods"
    ]
    """
    When you enable this parameter, this SetupIntent accepts payment methods that you enable in the Dashboard and that are compatible with its other parameters.
    """
    confirm: NotRequired[bool]
    """
    Set to `true` to attempt to confirm this SetupIntent immediately. This parameter defaults to `false`. If a card is the attached payment method, you can provide a `return_url` in case further authentication is necessary.
    """
    confirmation_token: NotRequired[str]
    """
    ID of the ConfirmationToken used to confirm this SetupIntent.

    If the provided ConfirmationToken contains properties that are also being provided in this request, such as `payment_method`, then the values in this request will take precedence.
    """
    customer: NotRequired[str]
    """
    ID of the Customer this SetupIntent belongs to, if one exists.

    If present, the SetupIntent's payment method will be attached to the Customer on successful setup. Payment methods attached to other Customers cannot be used with this SetupIntent.
    """
    description: NotRequired[str]
    """
    An arbitrary string attached to the object. Often useful for displaying to users.
    """
    excluded_payment_method_types: NotRequired[
        List[
            Literal[
                "acss_debit",
                "affirm",
                "afterpay_clearpay",
                "alipay",
                "alma",
                "amazon_pay",
                "au_becs_debit",
                "bacs_debit",
                "bancontact",
                "billie",
                "blik",
                "boleto",
                "card",
                "cashapp",
                "crypto",
                "customer_balance",
                "eps",
                "fpx",
                "giropay",
                "grabpay",
                "ideal",
                "kakao_pay",
                "klarna",
                "konbini",
                "kr_card",
                "mb_way",
                "mobilepay",
                "multibanco",
                "naver_pay",
                "nz_bank_account",
                "oxxo",
                "p24",
                "pay_by_bank",
                "payco",
                "paynow",
                "paypal",
                "pix",
                "promptpay",
                "revolut_pay",
                "samsung_pay",
                "satispay",
                "sepa_debit",
                "sofort",
                "swish",
                "twint",
                "us_bank_account",
                "wechat_pay",
                "zip",
            ]
        ]
    ]
    """
    The list of payment method types to exclude from use with this SetupIntent.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    flow_directions: NotRequired[List[Literal["inbound", "outbound"]]]
    """
    Indicates the directions of money movement for which this payment method is intended to be used.

    Include `inbound` if you intend to use the payment method as the origin to pull funds from. Include `outbound` if you intend to use the payment method as the destination to send funds to. You can include both if you intend to use the payment method for both purposes.
    """
    mandate_data: NotRequired["Literal['']|SetupIntentCreateParamsMandateData"]
    """
    This hash contains details about the mandate to create. This parameter can only be used with [`confirm=true`](https://stripe.com/docs/api/setup_intents/create#create_setup_intent-confirm).
    """
    metadata: NotRequired[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    on_behalf_of: NotRequired[str]
    """
    The Stripe account ID created for this SetupIntent.
    """
    payment_method: NotRequired[str]
    """
    ID of the payment method (a PaymentMethod, Card, or saved Source object) to attach to this SetupIntent.
    """
    payment_method_configuration: NotRequired[str]
    """
    The ID of the [payment method configuration](https://stripe.com/docs/api/payment_method_configurations) to use with this SetupIntent.
    """
    payment_method_data: NotRequired[
        "SetupIntentCreateParamsPaymentMethodData"
    ]
    """
    When included, this hash creates a PaymentMethod that is set as the [`payment_method`](https://stripe.com/docs/api/setup_intents/object#setup_intent_object-payment_method)
    value in the SetupIntent.
    """
    payment_method_options: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptions"
    ]
    """
    Payment method-specific configuration for this SetupIntent.
    """
    payment_method_types: NotRequired[List[str]]
    """
    The list of payment method types (for example, card) that this SetupIntent can use. If you don't provide this, Stripe will dynamically show relevant payment methods from your [payment method settings](https://dashboard.stripe.com/settings/payment_methods). A list of valid payment method types can be found [here](https://docs.stripe.com/api/payment_methods/object#payment_method_object-type).
    """
    return_url: NotRequired[str]
    """
    The URL to redirect your customer back to after they authenticate or cancel their payment on the payment method's app or site. To redirect to a mobile application, you can alternatively supply an application URI scheme. This parameter can only be used with [`confirm=true`](https://stripe.com/docs/api/setup_intents/create#create_setup_intent-confirm).
    """
    single_use: NotRequired["SetupIntentCreateParamsSingleUse"]
    """
    If you populate this hash, this SetupIntent generates a `single_use` mandate after successful completion.

    Single-use mandates are only valid for the following payment methods: `acss_debit`, `alipay`, `au_becs_debit`, `bacs_debit`, `bancontact`, `boleto`, `ideal`, `link`, `sepa_debit`, and `us_bank_account`.
    """
    usage: NotRequired[Literal["off_session", "on_session"]]
    """
    Indicates how the payment method is intended to be used in the future. If not provided, this value defaults to `off_session`.
    """
    use_stripe_sdk: NotRequired[bool]
    """
    Set to `true` when confirming server-side and using Stripe.js, iOS, or Android client-side SDKs to handle the next actions.
    """


class SetupIntentCreateParamsAutomaticPaymentMethods(TypedDict):
    allow_redirects: NotRequired[Literal["always", "never"]]
    """
    Controls whether this SetupIntent will accept redirect-based payment methods.

    Redirect-based payment methods may require your customer to be redirected to a payment method's app or site for authentication or additional steps. To [confirm](https://stripe.com/docs/api/setup_intents/confirm) this SetupIntent, you may be required to provide a `return_url` to redirect customers back to your site after they authenticate or complete the setup.
    """
    enabled: bool
    """
    Whether this feature is enabled.
    """


class SetupIntentCreateParamsMandateData(TypedDict):
    customer_acceptance: "SetupIntentCreateParamsMandateDataCustomerAcceptance"
    """
    This hash contains details about the customer acceptance of the Mandate.
    """


class SetupIntentCreateParamsMandateDataCustomerAcceptance(TypedDict):
    accepted_at: NotRequired[int]
    """
    The time at which the customer accepted the Mandate.
    """
    offline: NotRequired[
        "SetupIntentCreateParamsMandateDataCustomerAcceptanceOffline"
    ]
    """
    If this is a Mandate accepted offline, this hash contains details about the offline acceptance.
    """
    online: NotRequired[
        "SetupIntentCreateParamsMandateDataCustomerAcceptanceOnline"
    ]
    """
    If this is a Mandate accepted online, this hash contains details about the online acceptance.
    """
    type: Literal["offline", "online"]
    """
    The type of customer acceptance information included with the Mandate. One of `online` or `offline`.
    """


class SetupIntentCreateParamsMandateDataCustomerAcceptanceOffline(TypedDict):
    pass


class SetupIntentCreateParamsMandateDataCustomerAcceptanceOnline(TypedDict):
    ip_address: str
    """
    The IP address from which the Mandate was accepted by the customer.
    """
    user_agent: str
    """
    The user agent of the browser from which the Mandate was accepted by the customer.
    """


class SetupIntentCreateParamsPaymentMethodData(TypedDict):
    acss_debit: NotRequired[
        "SetupIntentCreateParamsPaymentMethodDataAcssDebit"
    ]
    """
    If this is an `acss_debit` PaymentMethod, this hash contains details about the ACSS Debit payment method.
    """
    affirm: NotRequired["SetupIntentCreateParamsPaymentMethodDataAffirm"]
    """
    If this is an `affirm` PaymentMethod, this hash contains details about the Affirm payment method.
    """
    afterpay_clearpay: NotRequired[
        "SetupIntentCreateParamsPaymentMethodDataAfterpayClearpay"
    ]
    """
    If this is an `AfterpayClearpay` PaymentMethod, this hash contains details about the AfterpayClearpay payment method.
    """
    alipay: NotRequired["SetupIntentCreateParamsPaymentMethodDataAlipay"]
    """
    If this is an `Alipay` PaymentMethod, this hash contains details about the Alipay payment method.
    """
    allow_redisplay: NotRequired[Literal["always", "limited", "unspecified"]]
    """
    This field indicates whether this payment method can be shown again to its customer in a checkout flow. Stripe products such as Checkout and Elements use this field to determine whether a payment method can be shown as a saved payment method in a checkout flow. The field defaults to `unspecified`.
    """
    alma: NotRequired["SetupIntentCreateParamsPaymentMethodDataAlma"]
    """
    If this is a Alma PaymentMethod, this hash contains details about the Alma payment method.
    """
    amazon_pay: NotRequired[
        "SetupIntentCreateParamsPaymentMethodDataAmazonPay"
    ]
    """
    If this is a AmazonPay PaymentMethod, this hash contains details about the AmazonPay payment method.
    """
    au_becs_debit: NotRequired[
        "SetupIntentCreateParamsPaymentMethodDataAuBecsDebit"
    ]
    """
    If this is an `au_becs_debit` PaymentMethod, this hash contains details about the bank account.
    """
    bacs_debit: NotRequired[
        "SetupIntentCreateParamsPaymentMethodDataBacsDebit"
    ]
    """
    If this is a `bacs_debit` PaymentMethod, this hash contains details about the Bacs Direct Debit bank account.
    """
    bancontact: NotRequired[
        "SetupIntentCreateParamsPaymentMethodDataBancontact"
    ]
    """
    If this is a `bancontact` PaymentMethod, this hash contains details about the Bancontact payment method.
    """
    billie: NotRequired["SetupIntentCreateParamsPaymentMethodDataBillie"]
    """
    If this is a `billie` PaymentMethod, this hash contains details about the Billie payment method.
    """
    billing_details: NotRequired[
        "SetupIntentCreateParamsPaymentMethodDataBillingDetails"
    ]
    """
    Billing information associated with the PaymentMethod that may be used or required by particular types of payment methods.
    """
    blik: NotRequired["SetupIntentCreateParamsPaymentMethodDataBlik"]
    """
    If this is a `blik` PaymentMethod, this hash contains details about the BLIK payment method.
    """
    boleto: NotRequired["SetupIntentCreateParamsPaymentMethodDataBoleto"]
    """
    If this is a `boleto` PaymentMethod, this hash contains details about the Boleto payment method.
    """
    cashapp: NotRequired["SetupIntentCreateParamsPaymentMethodDataCashapp"]
    """
    If this is a `cashapp` PaymentMethod, this hash contains details about the Cash App Pay payment method.
    """
    crypto: NotRequired["SetupIntentCreateParamsPaymentMethodDataCrypto"]
    """
    If this is a Crypto PaymentMethod, this hash contains details about the Crypto payment method.
    """
    customer_balance: NotRequired[
        "SetupIntentCreateParamsPaymentMethodDataCustomerBalance"
    ]
    """
    If this is a `customer_balance` PaymentMethod, this hash contains details about the CustomerBalance payment method.
    """
    eps: NotRequired["SetupIntentCreateParamsPaymentMethodDataEps"]
    """
    If this is an `eps` PaymentMethod, this hash contains details about the EPS payment method.
    """
    fpx: NotRequired["SetupIntentCreateParamsPaymentMethodDataFpx"]
    """
    If this is an `fpx` PaymentMethod, this hash contains details about the FPX payment method.
    """
    giropay: NotRequired["SetupIntentCreateParamsPaymentMethodDataGiropay"]
    """
    If this is a `giropay` PaymentMethod, this hash contains details about the Giropay payment method.
    """
    grabpay: NotRequired["SetupIntentCreateParamsPaymentMethodDataGrabpay"]
    """
    If this is a `grabpay` PaymentMethod, this hash contains details about the GrabPay payment method.
    """
    ideal: NotRequired["SetupIntentCreateParamsPaymentMethodDataIdeal"]
    """
    If this is an `ideal` PaymentMethod, this hash contains details about the iDEAL payment method.
    """
    interac_present: NotRequired[
        "SetupIntentCreateParamsPaymentMethodDataInteracPresent"
    ]
    """
    If this is an `interac_present` PaymentMethod, this hash contains details about the Interac Present payment method.
    """
    kakao_pay: NotRequired["SetupIntentCreateParamsPaymentMethodDataKakaoPay"]
    """
    If this is a `kakao_pay` PaymentMethod, this hash contains details about the Kakao Pay payment method.
    """
    klarna: NotRequired["SetupIntentCreateParamsPaymentMethodDataKlarna"]
    """
    If this is a `klarna` PaymentMethod, this hash contains details about the Klarna payment method.
    """
    konbini: NotRequired["SetupIntentCreateParamsPaymentMethodDataKonbini"]
    """
    If this is a `konbini` PaymentMethod, this hash contains details about the Konbini payment method.
    """
    kr_card: NotRequired["SetupIntentCreateParamsPaymentMethodDataKrCard"]
    """
    If this is a `kr_card` PaymentMethod, this hash contains details about the Korean Card payment method.
    """
    link: NotRequired["SetupIntentCreateParamsPaymentMethodDataLink"]
    """
    If this is an `Link` PaymentMethod, this hash contains details about the Link payment method.
    """
    mb_way: NotRequired["SetupIntentCreateParamsPaymentMethodDataMbWay"]
    """
    If this is a MB WAY PaymentMethod, this hash contains details about the MB WAY payment method.
    """
    metadata: NotRequired[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    mobilepay: NotRequired["SetupIntentCreateParamsPaymentMethodDataMobilepay"]
    """
    If this is a `mobilepay` PaymentMethod, this hash contains details about the MobilePay payment method.
    """
    multibanco: NotRequired[
        "SetupIntentCreateParamsPaymentMethodDataMultibanco"
    ]
    """
    If this is a `multibanco` PaymentMethod, this hash contains details about the Multibanco payment method.
    """
    naver_pay: NotRequired["SetupIntentCreateParamsPaymentMethodDataNaverPay"]
    """
    If this is a `naver_pay` PaymentMethod, this hash contains details about the Naver Pay payment method.
    """
    nz_bank_account: NotRequired[
        "SetupIntentCreateParamsPaymentMethodDataNzBankAccount"
    ]
    """
    If this is an nz_bank_account PaymentMethod, this hash contains details about the nz_bank_account payment method.
    """
    oxxo: NotRequired["SetupIntentCreateParamsPaymentMethodDataOxxo"]
    """
    If this is an `oxxo` PaymentMethod, this hash contains details about the OXXO payment method.
    """
    p24: NotRequired["SetupIntentCreateParamsPaymentMethodDataP24"]
    """
    If this is a `p24` PaymentMethod, this hash contains details about the P24 payment method.
    """
    pay_by_bank: NotRequired[
        "SetupIntentCreateParamsPaymentMethodDataPayByBank"
    ]
    """
    If this is a `pay_by_bank` PaymentMethod, this hash contains details about the PayByBank payment method.
    """
    payco: NotRequired["SetupIntentCreateParamsPaymentMethodDataPayco"]
    """
    If this is a `payco` PaymentMethod, this hash contains details about the PAYCO payment method.
    """
    paynow: NotRequired["SetupIntentCreateParamsPaymentMethodDataPaynow"]
    """
    If this is a `paynow` PaymentMethod, this hash contains details about the PayNow payment method.
    """
    paypal: NotRequired["SetupIntentCreateParamsPaymentMethodDataPaypal"]
    """
    If this is a `paypal` PaymentMethod, this hash contains details about the PayPal payment method.
    """
    pix: NotRequired["SetupIntentCreateParamsPaymentMethodDataPix"]
    """
    If this is a `pix` PaymentMethod, this hash contains details about the Pix payment method.
    """
    promptpay: NotRequired["SetupIntentCreateParamsPaymentMethodDataPromptpay"]
    """
    If this is a `promptpay` PaymentMethod, this hash contains details about the PromptPay payment method.
    """
    radar_options: NotRequired[
        "SetupIntentCreateParamsPaymentMethodDataRadarOptions"
    ]
    """
    Options to configure Radar. See [Radar Session](https://stripe.com/docs/radar/radar-session) for more information.
    """
    revolut_pay: NotRequired[
        "SetupIntentCreateParamsPaymentMethodDataRevolutPay"
    ]
    """
    If this is a `revolut_pay` PaymentMethod, this hash contains details about the Revolut Pay payment method.
    """
    samsung_pay: NotRequired[
        "SetupIntentCreateParamsPaymentMethodDataSamsungPay"
    ]
    """
    If this is a `samsung_pay` PaymentMethod, this hash contains details about the SamsungPay payment method.
    """
    satispay: NotRequired["SetupIntentCreateParamsPaymentMethodDataSatispay"]
    """
    If this is a `satispay` PaymentMethod, this hash contains details about the Satispay payment method.
    """
    sepa_debit: NotRequired[
        "SetupIntentCreateParamsPaymentMethodDataSepaDebit"
    ]
    """
    If this is a `sepa_debit` PaymentMethod, this hash contains details about the SEPA debit bank account.
    """
    sofort: NotRequired["SetupIntentCreateParamsPaymentMethodDataSofort"]
    """
    If this is a `sofort` PaymentMethod, this hash contains details about the SOFORT payment method.
    """
    swish: NotRequired["SetupIntentCreateParamsPaymentMethodDataSwish"]
    """
    If this is a `swish` PaymentMethod, this hash contains details about the Swish payment method.
    """
    twint: NotRequired["SetupIntentCreateParamsPaymentMethodDataTwint"]
    """
    If this is a TWINT PaymentMethod, this hash contains details about the TWINT payment method.
    """
    type: Literal[
        "acss_debit",
        "affirm",
        "afterpay_clearpay",
        "alipay",
        "alma",
        "amazon_pay",
        "au_becs_debit",
        "bacs_debit",
        "bancontact",
        "billie",
        "blik",
        "boleto",
        "cashapp",
        "crypto",
        "customer_balance",
        "eps",
        "fpx",
        "giropay",
        "grabpay",
        "ideal",
        "kakao_pay",
        "klarna",
        "konbini",
        "kr_card",
        "link",
        "mb_way",
        "mobilepay",
        "multibanco",
        "naver_pay",
        "nz_bank_account",
        "oxxo",
        "p24",
        "pay_by_bank",
        "payco",
        "paynow",
        "paypal",
        "pix",
        "promptpay",
        "revolut_pay",
        "samsung_pay",
        "satispay",
        "sepa_debit",
        "sofort",
        "swish",
        "twint",
        "us_bank_account",
        "wechat_pay",
        "zip",
    ]
    """
    The type of the PaymentMethod. An additional hash is included on the PaymentMethod with a name matching this value. It contains additional information specific to the PaymentMethod type.
    """
    us_bank_account: NotRequired[
        "SetupIntentCreateParamsPaymentMethodDataUsBankAccount"
    ]
    """
    If this is an `us_bank_account` PaymentMethod, this hash contains details about the US bank account payment method.
    """
    wechat_pay: NotRequired[
        "SetupIntentCreateParamsPaymentMethodDataWechatPay"
    ]
    """
    If this is an `wechat_pay` PaymentMethod, this hash contains details about the wechat_pay payment method.
    """
    zip: NotRequired["SetupIntentCreateParamsPaymentMethodDataZip"]
    """
    If this is a `zip` PaymentMethod, this hash contains details about the Zip payment method.
    """


class SetupIntentCreateParamsPaymentMethodDataAcssDebit(TypedDict):
    account_number: str
    """
    Customer's bank account number.
    """
    institution_number: str
    """
    Institution number of the customer's bank.
    """
    transit_number: str
    """
    Transit number of the customer's bank.
    """


class SetupIntentCreateParamsPaymentMethodDataAffirm(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataAfterpayClearpay(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataAlipay(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataAlma(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataAmazonPay(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataAuBecsDebit(TypedDict):
    account_number: str
    """
    The account number for the bank account.
    """
    bsb_number: str
    """
    Bank-State-Branch number of the bank account.
    """


class SetupIntentCreateParamsPaymentMethodDataBacsDebit(TypedDict):
    account_number: NotRequired[str]
    """
    Account number of the bank account that the funds will be debited from.
    """
    sort_code: NotRequired[str]
    """
    Sort code of the bank account. (e.g., `10-20-30`)
    """


class SetupIntentCreateParamsPaymentMethodDataBancontact(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataBillie(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataBillingDetails(TypedDict):
    address: NotRequired[
        "Literal['']|SetupIntentCreateParamsPaymentMethodDataBillingDetailsAddress"
    ]
    """
    Billing address.
    """
    email: NotRequired["Literal['']|str"]
    """
    Email address.
    """
    name: NotRequired["Literal['']|str"]
    """
    Full name.
    """
    phone: NotRequired["Literal['']|str"]
    """
    Billing phone number (including extension).
    """
    tax_id: NotRequired[str]
    """
    Taxpayer identification number. Used only for transactions between LATAM buyers and non-LATAM sellers.
    """


class SetupIntentCreateParamsPaymentMethodDataBillingDetailsAddress(TypedDict):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region.
    """


class SetupIntentCreateParamsPaymentMethodDataBlik(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataBoleto(TypedDict):
    tax_id: str
    """
    The tax ID of the customer (CPF for individual consumers or CNPJ for businesses consumers)
    """


class SetupIntentCreateParamsPaymentMethodDataCashapp(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataCrypto(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataCustomerBalance(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataEps(TypedDict):
    bank: NotRequired[
        Literal[
            "arzte_und_apotheker_bank",
            "austrian_anadi_bank_ag",
            "bank_austria",
            "bankhaus_carl_spangler",
            "bankhaus_schelhammer_und_schattera_ag",
            "bawag_psk_ag",
            "bks_bank_ag",
            "brull_kallmus_bank_ag",
            "btv_vier_lander_bank",
            "capital_bank_grawe_gruppe_ag",
            "deutsche_bank_ag",
            "dolomitenbank",
            "easybank_ag",
            "erste_bank_und_sparkassen",
            "hypo_alpeadriabank_international_ag",
            "hypo_bank_burgenland_aktiengesellschaft",
            "hypo_noe_lb_fur_niederosterreich_u_wien",
            "hypo_oberosterreich_salzburg_steiermark",
            "hypo_tirol_bank_ag",
            "hypo_vorarlberg_bank_ag",
            "marchfelder_bank",
            "oberbank_ag",
            "raiffeisen_bankengruppe_osterreich",
            "schoellerbank_ag",
            "sparda_bank_wien",
            "volksbank_gruppe",
            "volkskreditbank_ag",
            "vr_bank_braunau",
        ]
    ]
    """
    The customer's bank.
    """


class SetupIntentCreateParamsPaymentMethodDataFpx(TypedDict):
    account_holder_type: NotRequired[Literal["company", "individual"]]
    """
    Account holder type for FPX transaction
    """
    bank: Literal[
        "affin_bank",
        "agrobank",
        "alliance_bank",
        "ambank",
        "bank_islam",
        "bank_muamalat",
        "bank_of_china",
        "bank_rakyat",
        "bsn",
        "cimb",
        "deutsche_bank",
        "hong_leong_bank",
        "hsbc",
        "kfh",
        "maybank2e",
        "maybank2u",
        "ocbc",
        "pb_enterprise",
        "public_bank",
        "rhb",
        "standard_chartered",
        "uob",
    ]
    """
    The customer's bank.
    """


class SetupIntentCreateParamsPaymentMethodDataGiropay(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataGrabpay(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataIdeal(TypedDict):
    bank: NotRequired[
        Literal[
            "abn_amro",
            "asn_bank",
            "bunq",
            "buut",
            "finom",
            "handelsbanken",
            "ing",
            "knab",
            "moneyou",
            "n26",
            "nn",
            "rabobank",
            "regiobank",
            "revolut",
            "sns_bank",
            "triodos_bank",
            "van_lanschot",
            "yoursafe",
        ]
    ]
    """
    The customer's bank. Only use this parameter for existing customers. Don't use it for new customers.
    """


class SetupIntentCreateParamsPaymentMethodDataInteracPresent(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataKakaoPay(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataKlarna(TypedDict):
    dob: NotRequired["SetupIntentCreateParamsPaymentMethodDataKlarnaDob"]
    """
    Customer's date of birth
    """


class SetupIntentCreateParamsPaymentMethodDataKlarnaDob(TypedDict):
    day: int
    """
    The day of birth, between 1 and 31.
    """
    month: int
    """
    The month of birth, between 1 and 12.
    """
    year: int
    """
    The four-digit year of birth.
    """


class SetupIntentCreateParamsPaymentMethodDataKonbini(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataKrCard(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataLink(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataMbWay(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataMobilepay(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataMultibanco(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataNaverPay(TypedDict):
    funding: NotRequired[Literal["card", "points"]]
    """
    Whether to use Naver Pay points or a card to fund this transaction. If not provided, this defaults to `card`.
    """


class SetupIntentCreateParamsPaymentMethodDataNzBankAccount(TypedDict):
    account_holder_name: NotRequired[str]
    """
    The name on the bank account. Only required if the account holder name is different from the name of the authorized signatory collected in the PaymentMethod's billing details.
    """
    account_number: str
    """
    The account number for the bank account.
    """
    bank_code: str
    """
    The numeric code for the bank account's bank.
    """
    branch_code: str
    """
    The numeric code for the bank account's bank branch.
    """
    reference: NotRequired[str]
    suffix: str
    """
    The suffix of the bank account number.
    """


class SetupIntentCreateParamsPaymentMethodDataOxxo(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataP24(TypedDict):
    bank: NotRequired[
        Literal[
            "alior_bank",
            "bank_millennium",
            "bank_nowy_bfg_sa",
            "bank_pekao_sa",
            "banki_spbdzielcze",
            "blik",
            "bnp_paribas",
            "boz",
            "citi_handlowy",
            "credit_agricole",
            "envelobank",
            "etransfer_pocztowy24",
            "getin_bank",
            "ideabank",
            "ing",
            "inteligo",
            "mbank_mtransfer",
            "nest_przelew",
            "noble_pay",
            "pbac_z_ipko",
            "plus_bank",
            "santander_przelew24",
            "tmobile_usbugi_bankowe",
            "toyota_bank",
            "velobank",
            "volkswagen_bank",
        ]
    ]
    """
    The customer's bank.
    """


class SetupIntentCreateParamsPaymentMethodDataPayByBank(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataPayco(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataPaynow(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataPaypal(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataPix(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataPromptpay(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataRadarOptions(TypedDict):
    session: NotRequired[str]
    """
    A [Radar Session](https://stripe.com/docs/radar/radar-session) is a snapshot of the browser metadata and device details that help Radar make more accurate predictions on your payments.
    """


class SetupIntentCreateParamsPaymentMethodDataRevolutPay(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataSamsungPay(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataSatispay(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataSepaDebit(TypedDict):
    iban: str
    """
    IBAN of the bank account.
    """


class SetupIntentCreateParamsPaymentMethodDataSofort(TypedDict):
    country: Literal["AT", "BE", "DE", "ES", "IT", "NL"]
    """
    Two-letter ISO code representing the country the bank account is located in.
    """


class SetupIntentCreateParamsPaymentMethodDataSwish(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataTwint(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataUsBankAccount(TypedDict):
    account_holder_type: NotRequired[Literal["company", "individual"]]
    """
    Account holder type: individual or company.
    """
    account_number: NotRequired[str]
    """
    Account number of the bank account.
    """
    account_type: NotRequired[Literal["checking", "savings"]]
    """
    Account type: checkings or savings. Defaults to checking if omitted.
    """
    financial_connections_account: NotRequired[str]
    """
    The ID of a Financial Connections Account to use as a payment method.
    """
    routing_number: NotRequired[str]
    """
    Routing number of the bank account.
    """


class SetupIntentCreateParamsPaymentMethodDataWechatPay(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodDataZip(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodOptions(TypedDict):
    acss_debit: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptionsAcssDebit"
    ]
    """
    If this is a `acss_debit` SetupIntent, this sub-hash contains details about the ACSS Debit payment method options.
    """
    amazon_pay: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptionsAmazonPay"
    ]
    """
    If this is a `amazon_pay` SetupIntent, this sub-hash contains details about the AmazonPay payment method options.
    """
    bacs_debit: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptionsBacsDebit"
    ]
    """
    If this is a `bacs_debit` SetupIntent, this sub-hash contains details about the Bacs Debit payment method options.
    """
    card: NotRequired["SetupIntentCreateParamsPaymentMethodOptionsCard"]
    """
    Configuration for any card setup attempted on this SetupIntent.
    """
    card_present: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptionsCardPresent"
    ]
    """
    If this is a `card_present` PaymentMethod, this sub-hash contains details about the card-present payment method options.
    """
    klarna: NotRequired["SetupIntentCreateParamsPaymentMethodOptionsKlarna"]
    """
    If this is a `klarna` PaymentMethod, this hash contains details about the Klarna payment method options.
    """
    link: NotRequired["SetupIntentCreateParamsPaymentMethodOptionsLink"]
    """
    If this is a `link` PaymentMethod, this sub-hash contains details about the Link payment method options.
    """
    paypal: NotRequired["SetupIntentCreateParamsPaymentMethodOptionsPaypal"]
    """
    If this is a `paypal` PaymentMethod, this sub-hash contains details about the PayPal payment method options.
    """
    sepa_debit: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptionsSepaDebit"
    ]
    """
    If this is a `sepa_debit` SetupIntent, this sub-hash contains details about the SEPA Debit payment method options.
    """
    us_bank_account: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptionsUsBankAccount"
    ]
    """
    If this is a `us_bank_account` SetupIntent, this sub-hash contains details about the US bank account payment method options.
    """


class SetupIntentCreateParamsPaymentMethodOptionsAcssDebit(TypedDict):
    currency: NotRequired[Literal["cad", "usd"]]
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    mandate_options: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptionsAcssDebitMandateOptions"
    ]
    """
    Additional fields for Mandate creation
    """
    verification_method: NotRequired[
        Literal["automatic", "instant", "microdeposits"]
    ]
    """
    Bank account verification method.
    """


class SetupIntentCreateParamsPaymentMethodOptionsAcssDebitMandateOptions(
    TypedDict,
):
    custom_mandate_url: NotRequired["Literal['']|str"]
    """
    A URL for custom mandate text to render during confirmation step.
    The URL will be rendered with additional GET parameters `payment_intent` and `payment_intent_client_secret` when confirming a Payment Intent,
    or `setup_intent` and `setup_intent_client_secret` when confirming a Setup Intent.
    """
    default_for: NotRequired[List[Literal["invoice", "subscription"]]]
    """
    List of Stripe products where this mandate can be selected automatically.
    """
    interval_description: NotRequired[str]
    """
    Description of the mandate interval. Only required if 'payment_schedule' parameter is 'interval' or 'combined'.
    """
    payment_schedule: NotRequired[Literal["combined", "interval", "sporadic"]]
    """
    Payment schedule for the mandate.
    """
    transaction_type: NotRequired[Literal["business", "personal"]]
    """
    Transaction type of the mandate.
    """


class SetupIntentCreateParamsPaymentMethodOptionsAmazonPay(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodOptionsBacsDebit(TypedDict):
    mandate_options: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptionsBacsDebitMandateOptions"
    ]
    """
    Additional fields for Mandate creation
    """


class SetupIntentCreateParamsPaymentMethodOptionsBacsDebitMandateOptions(
    TypedDict,
):
    reference_prefix: NotRequired["Literal['']|str"]
    """
    Prefix used to generate the Mandate reference. Must be at most 12 characters long. Must consist of only uppercase letters, numbers, spaces, or the following special characters: '/', '_', '-', '&', '.'. Cannot begin with 'DDIC' or 'STRIPE'.
    """


class SetupIntentCreateParamsPaymentMethodOptionsCard(TypedDict):
    mandate_options: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptionsCardMandateOptions"
    ]
    """
    Configuration options for setting up an eMandate for cards issued in India.
    """
    moto: NotRequired[bool]
    """
    When specified, this parameter signals that a card has been collected
    as MOTO (Mail Order Telephone Order) and thus out of scope for SCA. This
    parameter can only be provided during confirmation.
    """
    network: NotRequired[
        Literal[
            "amex",
            "cartes_bancaires",
            "diners",
            "discover",
            "eftpos_au",
            "girocard",
            "interac",
            "jcb",
            "link",
            "mastercard",
            "unionpay",
            "unknown",
            "visa",
        ]
    ]
    """
    Selected network to process this SetupIntent on. Depends on the available networks of the card attached to the SetupIntent. Can be only set confirm-time.
    """
    request_three_d_secure: NotRequired[
        Literal["any", "automatic", "challenge"]
    ]
    """
    We strongly recommend that you rely on our SCA Engine to automatically prompt your customers for authentication based on risk level and [other requirements](https://stripe.com/docs/strong-customer-authentication). However, if you wish to request 3D Secure based on logic from your own fraud engine, provide this option. If not provided, this value defaults to `automatic`. Read our guide on [manually requesting 3D Secure](https://stripe.com/docs/payments/3d-secure/authentication-flow#manual-three-ds) for more information on how this configuration interacts with Radar and our SCA Engine.
    """
    three_d_secure: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptionsCardThreeDSecure"
    ]
    """
    If 3D Secure authentication was performed with a third-party provider,
    the authentication details to use for this setup.
    """


class SetupIntentCreateParamsPaymentMethodOptionsCardMandateOptions(TypedDict):
    amount: int
    """
    Amount to be charged for future payments.
    """
    amount_type: Literal["fixed", "maximum"]
    """
    One of `fixed` or `maximum`. If `fixed`, the `amount` param refers to the exact amount to be charged in future payments. If `maximum`, the amount charged can be up to the value passed for the `amount` param.
    """
    currency: str
    """
    Currency in which future payments will be charged. Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    description: NotRequired[str]
    """
    A description of the mandate or subscription that is meant to be displayed to the customer.
    """
    end_date: NotRequired[int]
    """
    End date of the mandate or subscription. If not provided, the mandate will be active until canceled. If provided, end date should be after start date.
    """
    interval: Literal["day", "month", "sporadic", "week", "year"]
    """
    Specifies payment frequency. One of `day`, `week`, `month`, `year`, or `sporadic`.
    """
    interval_count: NotRequired[int]
    """
    The number of intervals between payments. For example, `interval=month` and `interval_count=3` indicates one payment every three months. Maximum of one year interval allowed (1 year, 12 months, or 52 weeks). This parameter is optional when `interval=sporadic`.
    """
    reference: str
    """
    Unique identifier for the mandate or subscription.
    """
    start_date: int
    """
    Start date of the mandate or subscription. Start date should not be lesser than yesterday.
    """
    supported_types: NotRequired[List[Literal["india"]]]
    """
    Specifies the type of mandates supported. Possible values are `india`.
    """


class SetupIntentCreateParamsPaymentMethodOptionsCardThreeDSecure(TypedDict):
    ares_trans_status: NotRequired[Literal["A", "C", "I", "N", "R", "U", "Y"]]
    """
    The `transStatus` returned from the card Issuer's ACS in the ARes.
    """
    cryptogram: NotRequired[str]
    """
    The cryptogram, also known as the "authentication value" (AAV, CAVV or
    AEVV). This value is 20 bytes, base64-encoded into a 28-character string.
    (Most 3D Secure providers will return the base64-encoded version, which
    is what you should specify here.)
    """
    electronic_commerce_indicator: NotRequired[
        Literal["01", "02", "05", "06", "07"]
    ]
    """
    The Electronic Commerce Indicator (ECI) is returned by your 3D Secure
    provider and indicates what degree of authentication was performed.
    """
    network_options: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptionsCardThreeDSecureNetworkOptions"
    ]
    """
    Network specific 3DS fields. Network specific arguments require an
    explicit card brand choice. The parameter `payment_method_options.card.network``
    must be populated accordingly
    """
    requestor_challenge_indicator: NotRequired[str]
    """
    The challenge indicator (`threeDSRequestorChallengeInd`) which was requested in the
    AReq sent to the card Issuer's ACS. A string containing 2 digits from 01-99.
    """
    transaction_id: NotRequired[str]
    """
    For 3D Secure 1, the XID. For 3D Secure 2, the Directory Server
    Transaction ID (dsTransID).
    """
    version: NotRequired[Literal["1.0.2", "2.1.0", "2.2.0"]]
    """
    The version of 3D Secure that was performed.
    """


class SetupIntentCreateParamsPaymentMethodOptionsCardThreeDSecureNetworkOptions(
    TypedDict,
):
    cartes_bancaires: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptionsCardThreeDSecureNetworkOptionsCartesBancaires"
    ]
    """
    Cartes Bancaires-specific 3DS fields.
    """


class SetupIntentCreateParamsPaymentMethodOptionsCardThreeDSecureNetworkOptionsCartesBancaires(
    TypedDict,
):
    cb_avalgo: Literal["0", "1", "2", "3", "4", "A"]
    """
    The cryptogram calculation algorithm used by the card Issuer's ACS
    to calculate the Authentication cryptogram. Also known as `cavvAlgorithm`.
    messageExtension: CB-AVALGO
    """
    cb_exemption: NotRequired[str]
    """
    The exemption indicator returned from Cartes Bancaires in the ARes.
    message extension: CB-EXEMPTION; string (4 characters)
    This is a 3 byte bitmap (low significant byte first and most significant
    bit first) that has been Base64 encoded
    """
    cb_score: NotRequired[int]
    """
    The risk score returned from Cartes Bancaires in the ARes.
    message extension: CB-SCORE; numeric value 0-99
    """


class SetupIntentCreateParamsPaymentMethodOptionsCardPresent(TypedDict):
    pass


class SetupIntentCreateParamsPaymentMethodOptionsKlarna(TypedDict):
    currency: NotRequired[str]
    """
    The currency of the SetupIntent. Three letter ISO currency code.
    """
    on_demand: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptionsKlarnaOnDemand"
    ]
    """
    On-demand details if setting up a payment method for on-demand payments.
    """
    preferred_locale: NotRequired[
        Literal[
            "cs-CZ",
            "da-DK",
            "de-AT",
            "de-CH",
            "de-DE",
            "el-GR",
            "en-AT",
            "en-AU",
            "en-BE",
            "en-CA",
            "en-CH",
            "en-CZ",
            "en-DE",
            "en-DK",
            "en-ES",
            "en-FI",
            "en-FR",
            "en-GB",
            "en-GR",
            "en-IE",
            "en-IT",
            "en-NL",
            "en-NO",
            "en-NZ",
            "en-PL",
            "en-PT",
            "en-RO",
            "en-SE",
            "en-US",
            "es-ES",
            "es-US",
            "fi-FI",
            "fr-BE",
            "fr-CA",
            "fr-CH",
            "fr-FR",
            "it-CH",
            "it-IT",
            "nb-NO",
            "nl-BE",
            "nl-NL",
            "pl-PL",
            "pt-PT",
            "ro-RO",
            "sv-FI",
            "sv-SE",
        ]
    ]
    """
    Preferred language of the Klarna authorization page that the customer is redirected to
    """
    subscriptions: NotRequired[
        "Literal['']|List[SetupIntentCreateParamsPaymentMethodOptionsKlarnaSubscription]"
    ]
    """
    Subscription details if setting up or charging a subscription
    """


class SetupIntentCreateParamsPaymentMethodOptionsKlarnaOnDemand(TypedDict):
    average_amount: NotRequired[int]
    """
    Your average amount value. You can use a value across your customer base, or segment based on customer type, country, etc.
    """
    maximum_amount: NotRequired[int]
    """
    The maximum value you may charge a customer per purchase. You can use a value across your customer base, or segment based on customer type, country, etc.
    """
    minimum_amount: NotRequired[int]
    """
    The lowest or minimum value you may charge a customer per purchase. You can use a value across your customer base, or segment based on customer type, country, etc.
    """
    purchase_interval: NotRequired[Literal["day", "month", "week", "year"]]
    """
    Interval at which the customer is making purchases
    """
    purchase_interval_count: NotRequired[int]
    """
    The number of `purchase_interval` between charges
    """


class SetupIntentCreateParamsPaymentMethodOptionsKlarnaSubscription(TypedDict):
    interval: Literal["day", "month", "week", "year"]
    """
    Unit of time between subscription charges.
    """
    interval_count: NotRequired[int]
    """
    The number of intervals (specified in the `interval` attribute) between subscription charges. For example, `interval=month` and `interval_count=3` charges every 3 months.
    """
    name: NotRequired[str]
    """
    Name for subscription.
    """
    next_billing: "SetupIntentCreateParamsPaymentMethodOptionsKlarnaSubscriptionNextBilling"
    """
    Describes the upcoming charge for this subscription.
    """
    reference: str
    """
    A non-customer-facing reference to correlate subscription charges in the Klarna app. Use a value that persists across subscription charges.
    """


class SetupIntentCreateParamsPaymentMethodOptionsKlarnaSubscriptionNextBilling(
    TypedDict,
):
    amount: int
    """
    The amount of the next charge for the subscription.
    """
    date: str
    """
    The date of the next charge for the subscription in YYYY-MM-DD format.
    """


class SetupIntentCreateParamsPaymentMethodOptionsLink(TypedDict):
    persistent_token: NotRequired[str]
    """
    [Deprecated] This is a legacy parameter that no longer has any function.
    """


class SetupIntentCreateParamsPaymentMethodOptionsPaypal(TypedDict):
    billing_agreement_id: NotRequired[str]
    """
    The PayPal Billing Agreement ID (BAID). This is an ID generated by PayPal which represents the mandate between the merchant and the customer.
    """


class SetupIntentCreateParamsPaymentMethodOptionsSepaDebit(TypedDict):
    mandate_options: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptionsSepaDebitMandateOptions"
    ]
    """
    Additional fields for Mandate creation
    """


class SetupIntentCreateParamsPaymentMethodOptionsSepaDebitMandateOptions(
    TypedDict,
):
    reference_prefix: NotRequired["Literal['']|str"]
    """
    Prefix used to generate the Mandate reference. Must be at most 12 characters long. Must consist of only uppercase letters, numbers, spaces, or the following special characters: '/', '_', '-', '&', '.'. Cannot begin with 'STRIPE'.
    """


class SetupIntentCreateParamsPaymentMethodOptionsUsBankAccount(TypedDict):
    financial_connections: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptionsUsBankAccountFinancialConnections"
    ]
    """
    Additional fields for Financial Connections Session creation
    """
    mandate_options: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptionsUsBankAccountMandateOptions"
    ]
    """
    Additional fields for Mandate creation
    """
    networks: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptionsUsBankAccountNetworks"
    ]
    """
    Additional fields for network related functions
    """
    verification_method: NotRequired[
        Literal["automatic", "instant", "microdeposits"]
    ]
    """
    Bank account verification method.
    """


class SetupIntentCreateParamsPaymentMethodOptionsUsBankAccountFinancialConnections(
    TypedDict,
):
    filters: NotRequired[
        "SetupIntentCreateParamsPaymentMethodOptionsUsBankAccountFinancialConnectionsFilters"
    ]
    """
    Provide filters for the linked accounts that the customer can select for the payment method.
    """
    permissions: NotRequired[
        List[
            Literal["balances", "ownership", "payment_method", "transactions"]
        ]
    ]
    """
    The list of permissions to request. If this parameter is passed, the `payment_method` permission must be included. Valid permissions include: `balances`, `ownership`, `payment_method`, and `transactions`.
    """
    prefetch: NotRequired[
        List[Literal["balances", "ownership", "transactions"]]
    ]
    """
    List of data features that you would like to retrieve upon account creation.
    """
    return_url: NotRequired[str]
    """
    For webview integrations only. Upon completing OAuth login in the native browser, the user will be redirected to this URL to return to your app.
    """


class SetupIntentCreateParamsPaymentMethodOptionsUsBankAccountFinancialConnectionsFilters(
    TypedDict,
):
    account_subcategories: NotRequired[List[Literal["checking", "savings"]]]
    """
    The account subcategories to use to filter for selectable accounts. Valid subcategories are `checking` and `savings`.
    """


class SetupIntentCreateParamsPaymentMethodOptionsUsBankAccountMandateOptions(
    TypedDict,
):
    collection_method: NotRequired["Literal['']|Literal['paper']"]
    """
    The method used to collect offline mandate customer acceptance.
    """


class SetupIntentCreateParamsPaymentMethodOptionsUsBankAccountNetworks(
    TypedDict,
):
    requested: NotRequired[List[Literal["ach", "us_domestic_wire"]]]
    """
    Triggers validations to run across the selected networks
    """


class SetupIntentCreateParamsSingleUse(TypedDict):
    amount: int
    """
    Amount the customer is granting permission to collect later. A positive integer representing how much to charge in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal) (e.g., 100 cents to charge $1.00 or 100 to charge ¥100, a zero-decimal currency). The minimum amount is $0.50 US or [equivalent in charge currency](https://stripe.com/docs/currencies#minimum-and-maximum-charge-amounts). The amount value supports up to eight digits (e.g., a value of 99999999 for a USD charge of $999,999.99).
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.v2.billing._meter_event import MeterEvent as MeterEvent
    from stripe.v2.billing._meter_event_adjustment import (
        MeterEventAdjustment as MeterEventAdjustment,
    )
    from stripe.v2.billing._meter_event_adjustment_service import (
        MeterEventAdjustmentService as MeterEventAdjustmentService,
    )
    from stripe.v2.billing._meter_event_service import (
        MeterEventService as MeterEventService,
    )
    from stripe.v2.billing._meter_event_session import (
        MeterEventSession as MeterEventSession,
    )
    from stripe.v2.billing._meter_event_session_service import (
        MeterEventSessionService as MeterEventSessionService,
    )
    from stripe.v2.billing._meter_event_stream_service import (
        MeterEventStreamService as MeterEventStreamService,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "MeterEvent": ("stripe.v2.billing._meter_event", False),
    "MeterEventAdjustment": (
        "stripe.v2.billing._meter_event_adjustment",
        False,
    ),
    "MeterEventAdjustmentService": (
        "stripe.v2.billing._meter_event_adjustment_service",
        False,
    ),
    "MeterEventService": ("stripe.v2.billing._meter_event_service", False),
    "MeterEventSession": ("stripe.v2.billing._meter_event_session", False),
    "MeterEventSessionService": (
        "stripe.v2.billing._meter_event_session_service",
        False,
    ),
    "MeterEventStreamService": (
        "stripe.v2.billing._meter_event_stream_service",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import NotRequired


class ProductSearchParams(RequestOptions):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    limit: NotRequired[int]
    """
    A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
    """
    page: NotRequired[str]
    """
    A cursor for pagination across multiple pages of results. Don't include this parameter on the first call. Use the next_page value returned in a previous response to request subsequent results.
    """
    query: str
    """
    The search query string. See [search query language](https://stripe.com/docs/search#search-query-language) and the list of supported [query fields for products](https://stripe.com/docs/search#query-fields-for-products).
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.tax._association_find_params import (
        AssociationFindParams as AssociationFindParams,
    )
    from stripe.params.tax._calculation_create_params import (
        CalculationCreateParams as CalculationCreateParams,
        CalculationCreateParamsCustomerDetails as CalculationCreateParamsCustomerDetails,
        CalculationCreateParamsCustomerDetailsAddress as CalculationCreateParamsCustomerDetailsAddress,
        CalculationCreateParamsCustomerDetailsTaxId as CalculationCreateParamsCustomerDetailsTaxId,
        CalculationCreateParamsLineItem as CalculationCreateParamsLineItem,
        CalculationCreateParamsShipFromDetails as CalculationCreateParamsShipFromDetails,
        CalculationCreateParamsShipFromDetailsAddress as CalculationCreateParamsShipFromDetailsAddress,
        CalculationCreateParamsShippingCost as CalculationCreateParamsShippingCost,
    )
    from stripe.params.tax._calculation_line_item_list_params import (
        CalculationLineItemListParams as CalculationLineItemListParams,
    )
    from stripe.params.tax._calculation_list_line_items_params import (
        CalculationListLineItemsParams as CalculationListLineItemsParams,
    )
    from stripe.params.tax._calculation_retrieve_params import (
        CalculationRetrieveParams as CalculationRetrieveParams,
    )
    from stripe.params.tax._registration_create_params import (
        RegistrationCreateParams as RegistrationCreateParams,
        RegistrationCreateParamsCountryOptions as RegistrationCreateParamsCountryOptions,
        RegistrationCreateParamsCountryOptionsAe as RegistrationCreateParamsCountryOptionsAe,
        RegistrationCreateParamsCountryOptionsAeStandard as RegistrationCreateParamsCountryOptionsAeStandard,
        RegistrationCreateParamsCountryOptionsAl as RegistrationCreateParamsCountryOptionsAl,
        RegistrationCreateParamsCountryOptionsAlStandard as RegistrationCreateParamsCountryOptionsAlStandard,
        RegistrationCreateParamsCountryOptionsAm as RegistrationCreateParamsCountryOptionsAm,
        RegistrationCreateParamsCountryOptionsAo as RegistrationCreateParamsCountryOptionsAo,
        RegistrationCreateParamsCountryOptionsAoStandard as RegistrationCreateParamsCountryOptionsAoStandard,
        RegistrationCreateParamsCountryOptionsAt as RegistrationCreateParamsCountryOptionsAt,
        RegistrationCreateParamsCountryOptionsAtStandard as RegistrationCreateParamsCountryOptionsAtStandard,
        RegistrationCreateParamsCountryOptionsAu as RegistrationCreateParamsCountryOptionsAu,
        RegistrationCreateParamsCountryOptionsAuStandard as RegistrationCreateParamsCountryOptionsAuStandard,
        RegistrationCreateParamsCountryOptionsAw as RegistrationCreateParamsCountryOptionsAw,
        RegistrationCreateParamsCountryOptionsAwStandard as RegistrationCreateParamsCountryOptionsAwStandard,
        RegistrationCreateParamsCountryOptionsAz as RegistrationCreateParamsCountryOptionsAz,
        RegistrationCreateParamsCountryOptionsBa as RegistrationCreateParamsCountryOptionsBa,
        RegistrationCreateParamsCountryOptionsBaStandard as RegistrationCreateParamsCountryOptionsBaStandard,
        RegistrationCreateParamsCountryOptionsBb as RegistrationCreateParamsCountryOptionsBb,
        RegistrationCreateParamsCountryOptionsBbStandard as RegistrationCreateParamsCountryOptionsBbStandard,
        RegistrationCreateParamsCountryOptionsBd as RegistrationCreateParamsCountryOptionsBd,
        RegistrationCreateParamsCountryOptionsBdStandard as RegistrationCreateParamsCountryOptionsBdStandard,
        RegistrationCreateParamsCountryOptionsBe as RegistrationCreateParamsCountryOptionsBe,
        RegistrationCreateParamsCountryOptionsBeStandard as RegistrationCreateParamsCountryOptionsBeStandard,
        RegistrationCreateParamsCountryOptionsBf as RegistrationCreateParamsCountryOptionsBf,
        RegistrationCreateParamsCountryOptionsBfStandard as RegistrationCreateParamsCountryOptionsBfStandard,
        RegistrationCreateParamsCountryOptionsBg as RegistrationCreateParamsCountryOptionsBg,
        RegistrationCreateParamsCountryOptionsBgStandard as RegistrationCreateParamsCountryOptionsBgStandard,
        RegistrationCreateParamsCountryOptionsBh as RegistrationCreateParamsCountryOptionsBh,
        RegistrationCreateParamsCountryOptionsBhStandard as RegistrationCreateParamsCountryOptionsBhStandard,
        RegistrationCreateParamsCountryOptionsBj as RegistrationCreateParamsCountryOptionsBj,
        RegistrationCreateParamsCountryOptionsBs as RegistrationCreateParamsCountryOptionsBs,
        RegistrationCreateParamsCountryOptionsBsStandard as RegistrationCreateParamsCountryOptionsBsStandard,
        RegistrationCreateParamsCountryOptionsBy as RegistrationCreateParamsCountryOptionsBy,
        RegistrationCreateParamsCountryOptionsCa as RegistrationCreateParamsCountryOptionsCa,
        RegistrationCreateParamsCountryOptionsCaProvinceStandard as RegistrationCreateParamsCountryOptionsCaProvinceStandard,
        RegistrationCreateParamsCountryOptionsCd as RegistrationCreateParamsCountryOptionsCd,
        RegistrationCreateParamsCountryOptionsCdStandard as RegistrationCreateParamsCountryOptionsCdStandard,
        RegistrationCreateParamsCountryOptionsCh as RegistrationCreateParamsCountryOptionsCh,
        RegistrationCreateParamsCountryOptionsChStandard as RegistrationCreateParamsCountryOptionsChStandard,
        RegistrationCreateParamsCountryOptionsCl as RegistrationCreateParamsCountryOptionsCl,
        RegistrationCreateParamsCountryOptionsCm as RegistrationCreateParamsCountryOptionsCm,
        RegistrationCreateParamsCountryOptionsCo as RegistrationCreateParamsCountryOptionsCo,
        RegistrationCreateParamsCountryOptionsCr as RegistrationCreateParamsCountryOptionsCr,
        RegistrationCreateParamsCountryOptionsCv as RegistrationCreateParamsCountryOptionsCv,
        RegistrationCreateParamsCountryOptionsCy as RegistrationCreateParamsCountryOptionsCy,
        RegistrationCreateParamsCountryOptionsCyStandard as RegistrationCreateParamsCountryOptionsCyStandard,
        RegistrationCreateParamsCountryOptionsCz as RegistrationCreateParamsCountryOptionsCz,
        RegistrationCreateParamsCountryOptionsCzStandard as RegistrationCreateParamsCountryOptionsCzStandard,
        RegistrationCreateParamsCountryOptionsDe as RegistrationCreateParamsCountryOptionsDe,
        RegistrationCreateParamsCountryOptionsDeStandard as RegistrationCreateParamsCountryOptionsDeStandard,
        RegistrationCreateParamsCountryOptionsDk as RegistrationCreateParamsCountryOptionsDk,
        RegistrationCreateParamsCountryOptionsDkStandard as RegistrationCreateParamsCountryOptionsDkStandard,
        RegistrationCreateParamsCountryOptionsEc as RegistrationCreateParamsCountryOptionsEc,
        RegistrationCreateParamsCountryOptionsEe as RegistrationCreateParamsCountryOptionsEe,
        RegistrationCreateParamsCountryOptionsEeStandard as RegistrationCreateParamsCountryOptionsEeStandard,
        RegistrationCreateParamsCountryOptionsEg as RegistrationCreateParamsCountryOptionsEg,
        RegistrationCreateParamsCountryOptionsEs as RegistrationCreateParamsCountryOptionsEs,
        RegistrationCreateParamsCountryOptionsEsStandard as RegistrationCreateParamsCountryOptionsEsStandard,
        RegistrationCreateParamsCountryOptionsEt as RegistrationCreateParamsCountryOptionsEt,
        RegistrationCreateParamsCountryOptionsEtStandard as RegistrationCreateParamsCountryOptionsEtStandard,
        RegistrationCreateParamsCountryOptionsFi as RegistrationCreateParamsCountryOptionsFi,
        RegistrationCreateParamsCountryOptionsFiStandard as RegistrationCreateParamsCountryOptionsFiStandard,
        RegistrationCreateParamsCountryOptionsFr as RegistrationCreateParamsCountryOptionsFr,
        RegistrationCreateParamsCountryOptionsFrStandard as RegistrationCreateParamsCountryOptionsFrStandard,
        RegistrationCreateParamsCountryOptionsGb as RegistrationCreateParamsCountryOptionsGb,
        RegistrationCreateParamsCountryOptionsGbStandard as RegistrationCreateParamsCountryOptionsGbStandard,
        RegistrationCreateParamsCountryOptionsGe as RegistrationCreateParamsCountryOptionsGe,
        RegistrationCreateParamsCountryOptionsGn as RegistrationCreateParamsCountryOptionsGn,
        RegistrationCreateParamsCountryOptionsGnStandard as RegistrationCreateParamsCountryOptionsGnStandard,
        RegistrationCreateParamsCountryOptionsGr as RegistrationCreateParamsCountryOptionsGr,
        RegistrationCreateParamsCountryOptionsGrStandard as RegistrationCreateParamsCountryOptionsGrStandard,
        RegistrationCreateParamsCountryOptionsHr as RegistrationCreateParamsCountryOptionsHr,
        RegistrationCreateParamsCountryOptionsHrStandard as RegistrationCreateParamsCountryOptionsHrStandard,
        RegistrationCreateParamsCountryOptionsHu as RegistrationCreateParamsCountryOptionsHu,
        RegistrationCreateParamsCountryOptionsHuStandard as RegistrationCreateParamsCountryOptionsHuStandard,
        RegistrationCreateParamsCountryOptionsId as RegistrationCreateParamsCountryOptionsId,
        RegistrationCreateParamsCountryOptionsIe as RegistrationCreateParamsCountryOptionsIe,
        RegistrationCreateParamsCountryOptionsIeStandard as RegistrationCreateParamsCountryOptionsIeStandard,
        RegistrationCreateParamsCountryOptionsIn as RegistrationCreateParamsCountryOptionsIn,
        RegistrationCreateParamsCountryOptionsIs as RegistrationCreateParamsCountryOptionsIs,
        RegistrationCreateParamsCountryOptionsIsStandard as RegistrationCreateParamsCountryOptionsIsStandard,
        RegistrationCreateParamsCountryOptionsIt as RegistrationCreateParamsCountryOptionsIt,
        RegistrationCreateParamsCountryOptionsItStandard as RegistrationCreateParamsCountryOptionsItStandard,
        RegistrationCreateParamsCountryOptionsJp as RegistrationCreateParamsCountryOptionsJp,
        RegistrationCreateParamsCountryOptionsJpStandard as RegistrationCreateParamsCountryOptionsJpStandard,
        RegistrationCreateParamsCountryOptionsKe as RegistrationCreateParamsCountryOptionsKe,
        RegistrationCreateParamsCountryOptionsKg as RegistrationCreateParamsCountryOptionsKg,
        RegistrationCreateParamsCountryOptionsKh as RegistrationCreateParamsCountryOptionsKh,
        RegistrationCreateParamsCountryOptionsKr as RegistrationCreateParamsCountryOptionsKr,
        RegistrationCreateParamsCountryOptionsKz as RegistrationCreateParamsCountryOptionsKz,
        RegistrationCreateParamsCountryOptionsLa as RegistrationCreateParamsCountryOptionsLa,
        RegistrationCreateParamsCountryOptionsLt as RegistrationCreateParamsCountryOptionsLt,
        RegistrationCreateParamsCountryOptionsLtStandard as RegistrationCreateParamsCountryOptionsLtStandard,
        RegistrationCreateParamsCountryOptionsLu as RegistrationCreateParamsCountryOptionsLu,
        RegistrationCreateParamsCountryOptionsLuStandard as RegistrationCreateParamsCountryOptionsLuStandard,
        RegistrationCreateParamsCountryOptionsLv as RegistrationCreateParamsCountryOptionsLv,
        RegistrationCreateParamsCountryOptionsLvStandard as RegistrationCreateParamsCountryOptionsLvStandard,
        RegistrationCreateParamsCountryOptionsMa as RegistrationCreateParamsCountryOptionsMa,
        RegistrationCreateParamsCountryOptionsMd as RegistrationCreateParamsCountryOptionsMd,
        RegistrationCreateParamsCountryOptionsMe as RegistrationCreateParamsCountryOptionsMe,
        RegistrationCreateParamsCountryOptionsMeStandard as RegistrationCreateParamsCountryOptionsMeStandard,
        RegistrationCreateParamsCountryOptionsMk as RegistrationCreateParamsCountryOptionsMk,
        RegistrationCreateParamsCountryOptionsMkStandard as RegistrationCreateParamsCountryOptionsMkStandard,
        RegistrationCreateParamsCountryOptionsMr as RegistrationCreateParamsCountryOptionsMr,
        RegistrationCreateParamsCountryOptionsMrStandard as RegistrationCreateParamsCountryOptionsMrStandard,
        RegistrationCreateParamsCountryOptionsMt as RegistrationCreateParamsCountryOptionsMt,
        RegistrationCreateParamsCountryOptionsMtStandard as RegistrationCreateParamsCountryOptionsMtStandard,
        RegistrationCreateParamsCountryOptionsMx as RegistrationCreateParamsCountryOptionsMx,
        RegistrationCreateParamsCountryOptionsMy as RegistrationCreateParamsCountryOptionsMy,
        RegistrationCreateParamsCountryOptionsNg as RegistrationCreateParamsCountryOptionsNg,
        RegistrationCreateParamsCountryOptionsNl as RegistrationCreateParamsCountryOptionsNl,
        RegistrationCreateParamsCountryOptionsNlStandard as RegistrationCreateParamsCountryOptionsNlStandard,
        RegistrationCreateParamsCountryOptionsNo as RegistrationCreateParamsCountryOptionsNo,
        RegistrationCreateParamsCountryOptionsNoStandard as RegistrationCreateParamsCountryOptionsNoStandard,
        RegistrationCreateParamsCountryOptionsNp as RegistrationCreateParamsCountryOptionsNp,
        RegistrationCreateParamsCountryOptionsNz as RegistrationCreateParamsCountryOptionsNz,
        RegistrationCreateParamsCountryOptionsNzStandard as RegistrationCreateParamsCountryOptionsNzStandard,
        RegistrationCreateParamsCountryOptionsOm as RegistrationCreateParamsCountryOptionsOm,
        RegistrationCreateParamsCountryOptionsOmStandard as RegistrationCreateParamsCountryOptionsOmStandard,
        RegistrationCreateParamsCountryOptionsPe as RegistrationCreateParamsCountryOptionsPe,
        RegistrationCreateParamsCountryOptionsPh as RegistrationCreateParamsCountryOptionsPh,
        RegistrationCreateParamsCountryOptionsPl as RegistrationCreateParamsCountryOptionsPl,
        RegistrationCreateParamsCountryOptionsPlStandard as RegistrationCreateParamsCountryOptionsPlStandard,
        RegistrationCreateParamsCountryOptionsPt as RegistrationCreateParamsCountryOptionsPt,
        RegistrationCreateParamsCountryOptionsPtStandard as RegistrationCreateParamsCountryOptionsPtStandard,
        RegistrationCreateParamsCountryOptionsRo as RegistrationCreateParamsCountryOptionsRo,
        RegistrationCreateParamsCountryOptionsRoStandard as RegistrationCreateParamsCountryOptionsRoStandard,
        RegistrationCreateParamsCountryOptionsRs as RegistrationCreateParamsCountryOptionsRs,
        RegistrationCreateParamsCountryOptionsRsStandard as RegistrationCreateParamsCountryOptionsRsStandard,
        RegistrationCreateParamsCountryOptionsRu as RegistrationCreateParamsCountryOptionsRu,
        RegistrationCreateParamsCountryOptionsSa as RegistrationCreateParamsCountryOptionsSa,
        RegistrationCreateParamsCountryOptionsSe as RegistrationCreateParamsCountryOptionsSe,
        RegistrationCreateParamsCountryOptionsSeStandard as RegistrationCreateParamsCountryOptionsSeStandard,
        RegistrationCreateParamsCountryOptionsSg as RegistrationCreateParamsCountryOptionsSg,
        RegistrationCreateParamsCountryOptionsSgStandard as RegistrationCreateParamsCountryOptionsSgStandard,
        RegistrationCreateParamsCountryOptionsSi as RegistrationCreateParamsCountryOptionsSi,
        RegistrationCreateParamsCountryOptionsSiStandard as RegistrationCreateParamsCountryOptionsSiStandard,
        RegistrationCreateParamsCountryOptionsSk as RegistrationCreateParamsCountryOptionsSk,
        RegistrationCreateParamsCountryOptionsSkStandard as RegistrationCreateParamsCountryOptionsSkStandard,
        RegistrationCreateParamsCountryOptionsSn as RegistrationCreateParamsCountryOptionsSn,
        RegistrationCreateParamsCountryOptionsSr as RegistrationCreateParamsCountryOptionsSr,
        RegistrationCreateParamsCountryOptionsSrStandard as RegistrationCreateParamsCountryOptionsSrStandard,
        RegistrationCreateParamsCountryOptionsTh as RegistrationCreateParamsCountryOptionsTh,
        RegistrationCreateParamsCountryOptionsTj as RegistrationCreateParamsCountryOptionsTj,
        RegistrationCreateParamsCountryOptionsTr as RegistrationCreateParamsCountryOptionsTr,
        RegistrationCreateParamsCountryOptionsTw as RegistrationCreateParamsCountryOptionsTw,
        RegistrationCreateParamsCountryOptionsTz as RegistrationCreateParamsCountryOptionsTz,
        RegistrationCreateParamsCountryOptionsUa as RegistrationCreateParamsCountryOptionsUa,
        RegistrationCreateParamsCountryOptionsUg as RegistrationCreateParamsCountryOptionsUg,
        RegistrationCreateParamsCountryOptionsUs as RegistrationCreateParamsCountryOptionsUs,
        RegistrationCreateParamsCountryOptionsUsLocalAmusementTax as RegistrationCreateParamsCountryOptionsUsLocalAmusementTax,
        RegistrationCreateParamsCountryOptionsUsLocalLeaseTax as RegistrationCreateParamsCountryOptionsUsLocalLeaseTax,
        RegistrationCreateParamsCountryOptionsUsStateSalesTax as RegistrationCreateParamsCountryOptionsUsStateSalesTax,
        RegistrationCreateParamsCountryOptionsUsStateSalesTaxElection as RegistrationCreateParamsCountryOptionsUsStateSalesTaxElection,
        RegistrationCreateParamsCountryOptionsUy as RegistrationCreateParamsCountryOptionsUy,
        RegistrationCreateParamsCountryOptionsUyStandard as RegistrationCreateParamsCountryOptionsUyStandard,
        RegistrationCreateParamsCountryOptionsUz as RegistrationCreateParamsCountryOptionsUz,
        RegistrationCreateParamsCountryOptionsVn as RegistrationCreateParamsCountryOptionsVn,
        RegistrationCreateParamsCountryOptionsZa as RegistrationCreateParamsCountryOptionsZa,
        RegistrationCreateParamsCountryOptionsZaStandard as RegistrationCreateParamsCountryOptionsZaStandard,
        RegistrationCreateParamsCountryOptionsZm as RegistrationCreateParamsCountryOptionsZm,
        RegistrationCreateParamsCountryOptionsZw as RegistrationCreateParamsCountryOptionsZw,
        RegistrationCreateParamsCountryOptionsZwStandard as RegistrationCreateParamsCountryOptionsZwStandard,
    )
    from stripe.params.tax._registration_list_params import (
        RegistrationListParams as RegistrationListParams,
    )
    from stripe.params.tax._registration_modify_params import (
        RegistrationModifyParams as RegistrationModifyParams,
    )
    from stripe.params.tax._registration_retrieve_params import (
        RegistrationRetrieveParams as RegistrationRetrieveParams,
    )
    from stripe.params.tax._registration_update_params import (
        RegistrationUpdateParams as RegistrationUpdateParams,
    )
    from stripe.params.tax._settings_modify_params import (
        SettingsModifyParams as SettingsModifyParams,
        SettingsModifyParamsDefaults as SettingsModifyParamsDefaults,
        SettingsModifyParamsHeadOffice as SettingsModifyParamsHeadOffice,
        SettingsModifyParamsHeadOfficeAddress as SettingsModifyParamsHeadOfficeAddress,
    )
    from stripe.params.tax._settings_retrieve_params import (
        SettingsRetrieveParams as SettingsRetrieveParams,
    )
    from stripe.params.tax._settings_update_params import (
        SettingsUpdateParams as SettingsUpdateParams,
        SettingsUpdateParamsDefaults as SettingsUpdateParamsDefaults,
        SettingsUpdateParamsHeadOffice as SettingsUpdateParamsHeadOffice,
        SettingsUpdateParamsHeadOfficeAddress as SettingsUpdateParamsHeadOfficeAddress,
    )
    from stripe.params.tax._transaction_create_from_calculation_params import (
        TransactionCreateFromCalculationParams as TransactionCreateFromCalculationParams,
    )
    from stripe.params.tax._transaction_create_reversal_params import (
        TransactionCreateReversalParams as TransactionCreateReversalParams,
        TransactionCreateReversalParamsLineItem as TransactionCreateReversalParamsLineItem,
        TransactionCreateReversalParamsShippingCost as TransactionCreateReversalParamsShippingCost,
    )
    from stripe.params.tax._transaction_line_item_list_params import (
        TransactionLineItemListParams as TransactionLineItemListParams,
    )
    from stripe.params.tax._transaction_list_line_items_params import (
        TransactionListLineItemsParams as TransactionListLineItemsParams,
    )
    from stripe.params.tax._transaction_retrieve_params import (
        TransactionRetrieveParams as TransactionRetrieveParams,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "AssociationFindParams": (
        "stripe.params.tax._association_find_params",
        False,
    ),
    "CalculationCreateParams": (
        "stripe.params.tax._calculation_create_params",
        False,
    ),
    "CalculationCreateParamsCustomerDetails": (
        "stripe.params.tax._calculation_create_params",
        False,
    ),
    "CalculationCreateParamsCustomerDetailsAddress": (
        "stripe.params.tax._calculation_create_params",
        False,
    ),
    "CalculationCreateParamsCustomerDetailsTaxId": (
        "stripe.params.tax._calculation_create_params",
        False,
    ),
    "CalculationCreateParamsLineItem": (
        "stripe.params.tax._calculation_create_params",
        False,
    ),
    "CalculationCreateParamsShipFromDetails": (
        "stripe.params.tax._calculation_create_params",
        False,
    ),
    "CalculationCreateParamsShipFromDetailsAddress": (
        "stripe.params.tax._calculation_create_params",
        False,
    ),
    "CalculationCreateParamsShippingCost": (
        "stripe.params.tax._calculation_create_params",
        False,
    ),
    "CalculationLineItemListParams": (
        "stripe.params.tax._calculation_line_item_list_params",
        False,
    ),
    "CalculationListLineItemsParams": (
        "stripe.params.tax._calculation_list_line_items_params",
        False,
    ),
    "CalculationRetrieveParams": (
        "stripe.params.tax._calculation_retrieve_params",
        False,
    ),
    "RegistrationCreateParams": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptions": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsAe": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsAeStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsAl": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsAlStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsAm": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsAo": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsAoStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsAt": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsAtStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsAu": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsAuStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsAw": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsAwStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsAz": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsBa": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsBaStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsBb": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsBbStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsBd": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsBdStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsBe": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsBeStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsBf": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsBfStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsBg": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsBgStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsBh": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsBhStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsBj": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsBs": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsBsStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsBy": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsCa": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsCaProvinceStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsCd": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsCdStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsCh": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsChStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsCl": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsCm": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsCo": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsCr": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsCv": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsCy": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsCyStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsCz": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsCzStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsDe": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsDeStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsDk": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsDkStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsEc": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsEe": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsEeStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsEg": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsEs": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsEsStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsEt": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsEtStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsFi": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsFiStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsFr": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsFrStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsGb": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsGbStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsGe": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsGn": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsGnStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsGr": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsGrStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsHr": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsHrStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsHu": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsHuStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsId": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsIe": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsIeStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsIn": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsIs": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsIsStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsIt": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsItStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsJp": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsJpStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsKe": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsKg": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsKh": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsKr": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsKz": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsLa": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsLt": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsLtStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsLu": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsLuStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsLv": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsLvStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsMa": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsMd": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsMe": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsMeStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsMk": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsMkStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsMr": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsMrStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsMt": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsMtStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsMx": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsMy": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsNg": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsNl": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsNlStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsNo": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsNoStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsNp": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsNz": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsNzStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsOm": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsOmStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsPe": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsPh": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsPl": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsPlStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsPt": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsPtStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsRo": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsRoStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsRs": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsRsStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsRu": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsSa": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsSe": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsSeStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsSg": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsSgStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsSi": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsSiStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsSk": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsSkStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsSn": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsSr": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsSrStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsTh": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsTj": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsTr": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsTw": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsTz": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsUa": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsUg": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsUs": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsUsLocalAmusementTax": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsUsLocalLeaseTax": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsUsStateSalesTax": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsUsStateSalesTaxElection": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsUy": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsUyStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsUz": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsVn": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsZa": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsZaStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsZm": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsZw": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationCreateParamsCountryOptionsZwStandard": (
        "stripe.params.tax._registration_create_params",
        False,
    ),
    "RegistrationListParams": (
        "stripe.params.tax._registration_list_params",
        False,
    ),
    "RegistrationModifyParams": (
        "stripe.params.tax._registration_modify_params",
        False,
    ),
    "RegistrationRetrieveParams": (
        "stripe.params.tax._registration_retrieve_params",
        False,
    ),
    "RegistrationUpdateParams": (
        "stripe.params.tax._registration_update_params",
        False,
    ),
    "SettingsModifyParams": (
        "stripe.params.tax._settings_modify_params",
        False,
    ),
    "SettingsModifyParamsDefaults": (
        "stripe.params.tax._settings_modify_params",
        False,
    ),
    "SettingsModifyParamsHeadOffice": (
        "stripe.params.tax._settings_modify_params",
        False,
    ),
    "SettingsModifyParamsHeadOfficeAddress": (
        "stripe.params.tax._settings_modify_params",
        False,
    ),
    "SettingsRetrieveParams": (
        "stripe.params.tax._settings_retrieve_params",
        False,
    ),
    "SettingsUpdateParams": (
        "stripe.params.tax._settings_update_params",
        False,
    ),
    "SettingsUpdateParamsDefaults": (
        "stripe.params.tax._settings_update_params",
        False,
    ),
    "SettingsUpdateParamsHeadOffice": (
        "stripe.params.tax._settings_update_params",
        False,
    ),
    "SettingsUpdateParamsHeadOfficeAddress": (
        "stripe.params.tax._settings_update_params",
        False,
    ),
    "TransactionCreateFromCalculationParams": (
        "stripe.params.tax._transaction_create_from_calculation_params",
        False,
    ),
    "TransactionCreateReversalParams": (
        "stripe.params.tax._transaction_create_reversal_params",
        False,
    ),
    "TransactionCreateReversalParamsLineItem": (
        "stripe.params.tax._transaction_create_reversal_params",
        False,
    ),
    "TransactionCreateReversalParamsShippingCost": (
        "stripe.params.tax._transaction_create_reversal_params",
        False,
    ),
    "TransactionLineItemListParams": (
        "stripe.params.tax._transaction_line_item_list_params",
        False,
    ),
    "TransactionListLineItemsParams": (
        "stripe.params.tax._transaction_list_line_items_params",
        False,
    ),
    "TransactionRetrieveParams": (
        "stripe.params.tax._transaction_retrieve_params",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



import stripe


TEST_RESOURCE_ID = "sub_123"


class TestSubscription(object):
    def test_is_listable(self, http_client_mock):
        resources = stripe.Subscription.list()
        http_client_mock.assert_requested("get", path="/v1/subscriptions")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], stripe.Subscription)

    def test_is_retrievable(self, http_client_mock):
        resource = stripe.Subscription.retrieve(TEST_RESOURCE_ID)
        http_client_mock.assert_requested(
            "get", path="/v1/subscriptions/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Subscription)

    def test_is_creatable(self, http_client_mock):
        resource = stripe.Subscription.create(customer="cus_123")
        http_client_mock.assert_requested(
            "post", path="/v1/subscriptions", post_data="customer=cus_123"
        )
        assert isinstance(resource, stripe.Subscription)

    def test_is_saveable(self, http_client_mock):
        resource = stripe.Subscription.retrieve(TEST_RESOURCE_ID)
        resource.metadata["key"] = "value"
        resource.save()
        http_client_mock.assert_requested(
            "post",
            path="/v1/subscriptions/%s" % TEST_RESOURCE_ID,
            post_data="metadata[key]=value",
        )

    def test_is_modifiable(self, http_client_mock):
        resource = stripe.Subscription.modify(
            TEST_RESOURCE_ID, metadata={"key": "value"}
        )
        http_client_mock.assert_requested(
            "post",
            path="/v1/subscriptions/%s" % TEST_RESOURCE_ID,
            post_data="metadata[key]=value",
        )
        assert isinstance(resource, stripe.Subscription)

    def test_is_deletable(self, http_client_mock):
        resource = stripe.Subscription.retrieve(TEST_RESOURCE_ID)
        resource.delete()
        http_client_mock.assert_requested(
            "delete", path="/v1/subscriptions/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Subscription)

    def test_can_delete(self, http_client_mock):
        resource = stripe.Subscription.delete(TEST_RESOURCE_ID)
        http_client_mock.assert_requested(
            "delete", path="/v1/subscriptions/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Subscription)

    def test_can_delete_discount(self, http_client_mock):
        sub = stripe.Subscription.retrieve(TEST_RESOURCE_ID)
        sub.delete_discount()
        http_client_mock.assert_requested(
            "delete", path="/v1/subscriptions/%s/discount" % sub.id
        )

    def test_can_delete_discount_classmethod(self, http_client_mock):
        stripe.Subscription.delete_discount(TEST_RESOURCE_ID)
        http_client_mock.assert_requested(
            "delete", path="/v1/subscriptions/%s/discount" % TEST_RESOURCE_ID
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class Application(StripeObject):
    OBJECT_NAME: ClassVar[Literal["application"]] = "application"
    deleted: Optional[Literal[True]]
    """
    Always true for a deleted object
    """
    id: str
    """
    Unique identifier for the object.
    """
    name: Optional[str]
    """
    The name of the application.
    """
    object: Literal["application"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """



import stripe


TEST_RESOURCE_ID = "pi_123"


class TestPaymentIntent(object):
    def test_is_listable(self, http_client_mock):
        resources = stripe.PaymentIntent.list()
        http_client_mock.assert_requested("get", path="/v1/payment_intents")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], stripe.PaymentIntent)

    def test_is_retrievable(self, http_client_mock):
        resource = stripe.PaymentIntent.retrieve(TEST_RESOURCE_ID)
        http_client_mock.assert_requested(
            "get", path="/v1/payment_intents/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.PaymentIntent)

    def test_is_creatable(self, http_client_mock):
        resource = stripe.PaymentIntent.create(
            amount="1234", currency="amount", payment_method_types=["card"]
        )
        http_client_mock.assert_requested(
            "post",
            path="/v1/payment_intents",
            post_data="amount=1234&currency=amount&payment_method_types[0]=card",
        )
        assert isinstance(resource, stripe.PaymentIntent)

    def test_is_modifiable(self, http_client_mock):
        resource = stripe.PaymentIntent.modify(
            TEST_RESOURCE_ID, metadata={"key": "value"}
        )
        http_client_mock.assert_requested(
            "post",
            path="/v1/payment_intents/%s" % TEST_RESOURCE_ID,
            post_data="metadata[key]=value",
        )
        assert isinstance(resource, stripe.PaymentIntent)

    def test_is_saveable(self, http_client_mock):
        resource = stripe.PaymentIntent.retrieve(TEST_RESOURCE_ID)

        resource.metadata["key"] = "value"
        resource.save()
        http_client_mock.assert_requested(
            "post",
            path="/v1/payment_intents/%s" % TEST_RESOURCE_ID,
            post_data="metadata[key]=value",
        )
        assert isinstance(resource, stripe.PaymentIntent)

    def test_can_cancel(self, http_client_mock):
        resource = stripe.PaymentIntent.retrieve(TEST_RESOURCE_ID)
        resource.cancel()
        http_client_mock.assert_requested(
            "post", path="/v1/payment_intents/%s/cancel" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.PaymentIntent)

    def test_can_cancel_classmethod(self, http_client_mock):
        resource = stripe.PaymentIntent.cancel(TEST_RESOURCE_ID)
        http_client_mock.assert_requested(
            "post", path="/v1/payment_intents/%s/cancel" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.PaymentIntent)

    def test_can_capture(self, http_client_mock):
        resource = stripe.PaymentIntent.retrieve(TEST_RESOURCE_ID)
        resource.capture()
        http_client_mock.assert_requested(
            "post", path="/v1/payment_intents/%s/capture" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.PaymentIntent)

    def test_can_capture_classmethod(self, http_client_mock):
        resource = stripe.PaymentIntent.capture(TEST_RESOURCE_ID)
        http_client_mock.assert_requested(
            "post", path="/v1/payment_intents/%s/capture" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.PaymentIntent)

    def test_can_confirm(self, http_client_mock):
        resource = stripe.PaymentIntent.retrieve(TEST_RESOURCE_ID)
        resource.confirm()
        http_client_mock.assert_requested(
            "post", path="/v1/payment_intents/%s/confirm" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.PaymentIntent)

    def test_can_confirm_classmethod(self, http_client_mock):
        resource = stripe.PaymentIntent.confirm(TEST_RESOURCE_ID)
        http_client_mock.assert_requested(
            "post", path="/v1/payment_intents/%s/confirm" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.PaymentIntent)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe._shipping_rate import ShippingRate
    from stripe.params._shipping_rate_create_params import (
        ShippingRateCreateParams,
    )
    from stripe.params._shipping_rate_list_params import ShippingRateListParams
    from stripe.params._shipping_rate_retrieve_params import (
        ShippingRateRetrieveParams,
    )
    from stripe.params._shipping_rate_update_params import (
        ShippingRateUpdateParams,
    )


class ShippingRateService(StripeService):
    def list(
        self,
        params: Optional["ShippingRateListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[ShippingRate]":
        """
        Returns a list of your shipping rates.
        """
        return cast(
            "ListObject[ShippingRate]",
            self._request(
                "get",
                "/v1/shipping_rates",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["ShippingRateListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[ShippingRate]":
        """
        Returns a list of your shipping rates.
        """
        return cast(
            "ListObject[ShippingRate]",
            await self._request_async(
                "get",
                "/v1/shipping_rates",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "ShippingRateCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ShippingRate":
        """
        Creates a new shipping rate object.
        """
        return cast(
            "ShippingRate",
            self._request(
                "post",
                "/v1/shipping_rates",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "ShippingRateCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ShippingRate":
        """
        Creates a new shipping rate object.
        """
        return cast(
            "ShippingRate",
            await self._request_async(
                "post",
                "/v1/shipping_rates",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        shipping_rate_token: str,
        params: Optional["ShippingRateRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ShippingRate":
        """
        Returns the shipping rate object with the given ID.
        """
        return cast(
            "ShippingRate",
            self._request(
                "get",
                "/v1/shipping_rates/{shipping_rate_token}".format(
                    shipping_rate_token=sanitize_id(shipping_rate_token),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        shipping_rate_token: str,
        params: Optional["ShippingRateRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ShippingRate":
        """
        Returns the shipping rate object with the given ID.
        """
        return cast(
            "ShippingRate",
            await self._request_async(
                "get",
                "/v1/shipping_rates/{shipping_rate_token}".format(
                    shipping_rate_token=sanitize_id(shipping_rate_token),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        shipping_rate_token: str,
        params: Optional["ShippingRateUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ShippingRate":
        """
        Updates an existing shipping rate object.
        """
        return cast(
            "ShippingRate",
            self._request(
                "post",
                "/v1/shipping_rates/{shipping_rate_token}".format(
                    shipping_rate_token=sanitize_id(shipping_rate_token),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        shipping_rate_token: str,
        params: Optional["ShippingRateUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ShippingRate":
        """
        Updates an existing shipping rate object.
        """
        return cast(
            "ShippingRate",
            await self._request_async(
                "post",
                "/v1/shipping_rates/{shipping_rate_token}".format(
                    shipping_rate_token=sanitize_id(shipping_rate_token),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params.tax._calculation_line_item_list_params import (
        CalculationLineItemListParams,
    )
    from stripe.tax._calculation_line_item import CalculationLineItem


class CalculationLineItemService(StripeService):
    def list(
        self,
        calculation: str,
        params: Optional["CalculationLineItemListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[CalculationLineItem]":
        """
        Retrieves the line items of a tax calculation as a collection, if the calculation hasn't expired.
        """
        return cast(
            "ListObject[CalculationLineItem]",
            self._request(
                "get",
                "/v1/tax/calculations/{calculation}/line_items".format(
                    calculation=sanitize_id(calculation),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        calculation: str,
        params: Optional["CalculationLineItemListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[CalculationLineItem]":
        """
        Retrieves the line items of a tax calculation as a collection, if the calculation hasn't expired.
        """
        return cast(
            "ListObject[CalculationLineItem]",
            await self._request_async(
                "get",
                "/v1/tax/calculations/{calculation}/line_items".format(
                    calculation=sanitize_id(calculation),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



from __future__ import absolute_import, division, print_function

import datetime

import stripe

from tests.test_api_requestor import GMT1


class TestRawRequest(object):
    ENCODE_INPUTS = {
        "type": "standard",
        "int": 123,
        "datetime": datetime.datetime(2013, 1, 1, second=1, tzinfo=GMT1()),
    }
    POST_REL_URL = "/v1/accounts"
    GET_REL_URL = "/v1/accounts/acct_123"
    POST_REL_URL_V2 = "/v2/billing/meter_event_session"
    GET_REL_URL_V2 = "/v2/accounts/acct_123"

    def test_form_request_get(
        self, http_client_mock, stripe_mock_stripe_client
    ):
        http_client_mock.stub_request(
            "get",
            path=self.GET_REL_URL,
            rbody='{"id": "acct_123", "object": "account"}',
            rcode=200,
            rheaders={},
        )

        resp = stripe_mock_stripe_client.raw_request("get", self.GET_REL_URL)
        http_client_mock.assert_requested("get", path=self.GET_REL_URL)

        deserialized = stripe_mock_stripe_client.deserialize(
            resp, api_mode="V1"
        )
        assert isinstance(deserialized, stripe.Account)

    def test_form_request_post(
        self, http_client_mock, stripe_mock_stripe_client
    ):
        http_client_mock.stub_request(
            "post",
            path=self.POST_REL_URL,
            rbody='{"id": "acct_123", "object": "account"}',
            rcode=200,
            rheaders={},
        )

        expectation = "type=standard&int=123&datetime=1356994801"

        resp = stripe_mock_stripe_client.raw_request(
            "post", self.POST_REL_URL, **self.ENCODE_INPUTS
        )

        http_client_mock.assert_requested(
            "post",
            path=self.POST_REL_URL,
            content_type="application/x-www-form-urlencoded",
            post_data=expectation,
        )

        deserialized = stripe_mock_stripe_client.deserialize(
            resp, api_mode="V1"
        )
        assert isinstance(deserialized, stripe.Account)

    def test_v2_request_post(
        self, http_client_mock, stripe_mock_stripe_client
    ):
        http_client_mock.stub_request(
            "post",
            path=self.POST_REL_URL_V2,
            rbody='{"id": "bmes_123", "object": "v2.billing.meter_event_session"}',
            rcode=200,
            rheaders={},
        )

        params = dict({}, **self.ENCODE_INPUTS)
        expectation = (
            '{"type": "standard", "int": 123, "datetime": 1356994801}'
        )

        resp = stripe_mock_stripe_client.raw_request(
            "post", self.POST_REL_URL_V2, **params
        )

        http_client_mock.assert_requested(
            "post",
            path=self.POST_REL_URL_V2,
            content_type="application/json",
            post_data=expectation,
            is_json=True,
        )

        deserialized = stripe_mock_stripe_client.deserialize(
            resp, api_mode="V2"
        )
        assert isinstance(deserialized, stripe.v2.billing.MeterEventSession)

    def test_form_request_with_extra_headers(
        self, http_client_mock, stripe_mock_stripe_client
    ):
        http_client_mock.stub_request(
            "get",
            path=self.GET_REL_URL,
            rbody='{"id": "acct_123", "object": "account"}',
            rcode=200,
            rheaders={},
        )

        extra_headers = {"foo": "bar", "Stripe-Account": "acct_123"}
        params = {"headers": extra_headers}

        stripe_mock_stripe_client.raw_request(
            "get", self.GET_REL_URL, **params
        )

        http_client_mock.assert_requested(
            "get",
            path=self.GET_REL_URL,
            extra_headers=extra_headers,
        )

    def test_v2_request_default_api_version(
        self, http_client_mock, stripe_mock_stripe_client
    ):
        http_client_mock.stub_request(
            "get",
            path=self.GET_REL_URL_V2,
            rbody='{"id": "acct_123", "object": "account"}',
            rcode=200,
            rheaders={},
        )
        params = {}

        stripe_mock_stripe_client.raw_request(
            "get", self.GET_REL_URL_V2, **params
        )

        http_client_mock.assert_requested(
            "get",
            path=self.GET_REL_URL_V2,
        )

    def test_v2_request_overridden_api_version(
        self, http_client_mock, stripe_mock_stripe_client
    ):
        http_client_mock.stub_request(
            "post",
            path=self.POST_REL_URL_V2,
            rbody='{"id": "acct_123", "object": "account"}',
            rcode=200,
            rheaders={},
        )
        stripe_version_override = "2023-05-15.preview"
        params = {
            "stripe_version": stripe_version_override,
        }

        stripe_mock_stripe_client.raw_request(
            "post", self.POST_REL_URL_V2, **params
        )

        http_client_mock.assert_requested(
            "post",
            path=self.POST_REL_URL_V2,
            content_type="application/json",
            stripe_version=stripe_version_override,
            post_data="{}",
            is_json=True,
        )

    # TODO(jar) this test is not applicable yet, but may be some day
    #    @pytest.mark.anyio
    #    async def test_form_request_get_async(
    #        self, http_client_mock, stripe_mock_stripe_client
    #    ):
    #        http_client_mock.stub_request(
    #            "get",
    #            path=self.GET_REL_URL,
    #            rbody='{"id": "acct_123", "object": "account"}',
    #            rcode=200,
    #            rheaders={},
    #        )
    #
    #        resp = await stripe_mock_stripe_client.raw_request_async(
    #            "get", self.GET_REL_URL
    #        )
    #
    #        http_client_mock.assert_requested("get", path=self.GET_REL_URL)
    #
    #        deserialized = stripe_mock_stripe_client.deserialize(resp)
    #        assert isinstance(deserialized, stripe.Account)
    #
    def test_raw_request_usage_reported(
        self, http_client_mock, stripe_mock_stripe_client
    ):
        http_client_mock.stub_request(
            "post",
            path=self.POST_REL_URL,
            rbody='{"id": "acct_123", "object": "account"}',
            rcode=200,
            rheaders={},
        )

        expectation = "type=standard&int=123&datetime=1356994801"

        resp = stripe_mock_stripe_client.raw_request(
            "post", self.POST_REL_URL, **self.ENCODE_INPUTS
        )

        http_client_mock.assert_requested(
            "post",
            path=self.POST_REL_URL,
            content_type="application/x-www-form-urlencoded",
            post_data=expectation,
            usage=["raw_request"],
        )

        deserialized = stripe_mock_stripe_client.deserialize(
            resp, api_mode="V1"
        )
        assert isinstance(deserialized, stripe.Account)
